import pygame
import classes.board
import random
import colorsys
import os
import classes.level_controller

class Layout:
    def __init__(self,mainloop, screen_w, screen_h,x_count=26,y_count=11):
        self.screen_w = screen_w
        self.screen_h = screen_h   
        self.mainloop = mainloop     
        self.update_layout(x_count,y_count)

    def update_layout(self,x_count,y_count):
        self.x_count = x_count
        self.y_count = y_count

        self.menu_w = 124+5 #+5 - extra space to make the gap for tabs to look ok
        #50+10+50+10+1
        self.grid_line_w = 1

        self.info_bar_h = 76
        self.info_bar_offset_h = 76
        
        self.avail_game_w = self.screen_w - self.menu_w  
        self.avail_game_h = self.screen_h - self.info_bar_h
        
        #find the right size (scale) for a single square and calculate panels' sizes
        scale_x = (self.screen_w - self.menu_w - self.grid_line_w) // x_count
        scale_y = (self.screen_h - self.info_bar_h - self.grid_line_w) // y_count

        if scale_x < scale_y:
            self.scale = scale_x
        else:
            self.scale = scale_y

        self.menu_w += (self.screen_w - self.menu_w) - self.scale*x_count - self.grid_line_w#(screen_w - menu_w) % x_count
        self.menu_l_w = 62
        self.menu_r_w = 62 #self.menu_w - self.menu_l_w

        self.width=self.scale #width of a single square
        self.height=self.scale
        self.game_h = y_count*self.height+self.grid_line_w
        self.game_w = self.screen_w - self.menu_w
        self.info_bar_offset_h += (self.screen_h - self.info_bar_offset_h) - self.game_h
        self.menu_pos = (0,0, self.menu_w, self.screen_h)
        self.menu_l_pos = (0,0, self.menu_l_w, self.screen_h)
        self.menu_r_pos = (self.menu_l_w,0, self.menu_r_w, self.screen_h)
        self.game_pos = (self.menu_w,0, self.game_w, self.game_h) #changed
        #self.misio_pos = (0,self.screen_h-179, 140, 179)
        #self.misio_pos = (0,self.screen_h-123, 112, 123)
        self.misio_pos = (0,0, 123, 123)
        self.info_bar_offset_pos = (self.menu_w, self.game_h, self.game_w, self.info_bar_offset_h)
        self.info_bar_pos = (1, self.info_bar_offset_h - self.info_bar_h, self.game_w-1, self.info_bar_h)
        self.info_top = self.game_h + self.info_bar_pos[1]
        
    def draw_layout(self):
        pass
        
    def update_layout_fs(self,screen_w, screen_h,x_count,y_count):
        #update layout after switching from fullscreen to windowed view
        self.__init__(self.mainloop, screen_w, screen_h,x_count,y_count)
        self.mainloop.m.update_scroll_pos()
    
class Dialog:
    def __init__(self,game_board):
        self.lines = ["CONGRATULATIONS!!!", "YOU HAVE COMPLETED", "THIS LEVEL"]
        self.lines_game_over = ["GAME OVER!!!"]
        self.game_board = game_board
        self.color = (70,70,70)
        self.img_src = "congrats.png"
        self.img_src2 = "game_over.png"   
        points = int(round((60 * 72 /96)*1.5,0))
        self.font = pygame.font.Font(None, (points))
        self.layout = game_board.layout
        self.layout_update()
        self.level = game_board.level
        
    def layout_update(self):
        self.width = self.layout.game_w - 1
        self.height = self.layout.game_h
        self.image = pygame.Surface([self.width, self.height])
        self.image.fill(self.color)
        self.rect = self.image.get_rect()
        self.rect.topleft = [0, 0]
        self.img = pygame.image.load(os.path.join('images', self.img_src)).convert()
        self.img2 = pygame.image.load(os.path.join('images', self.img_src2)).convert()
        
        #img2 has the same size
        img_pos_x = self.img.get_rect(centerx=self.image.get_width()//2)
        img_pos_y = self.img.get_rect(centery=self.image.get_height()//2)
        self.img_pos = (img_pos_x[0],img_pos_y[1])
    
    def image_swap(self,img_src):
        pass

        
    def display_lines(self, lines):
        y_pos = 190
        for each in lines:
            text = self.font.render("%s" % (each), 1, (0, 0, 0, 0))
            textpos = text.get_rect(centerx=self.image.get_width()//2)
            self.image.fill((255,255,255))
            self.image.blit(text, (textpos[0],y_pos))
            y_pos += 70

            """
    def display_game_over(self):
        y_pos = 190
        for each in self.lines_game_over:
            text = self.font.render("%s" % (each), 1, (255, 255, 255, 0))
            textpos = text.get_rect(centerx=self.image.get_width()//2)
            self.image.blit(text, (textpos[0],y_pos))
            y_pos += 70
"""

    def update(self,screen):
        self.image.fill(self.color)
        if self.level.dialog_type == 0:
            self.image.blit(self.img, (self.img_pos))
            
        elif  self.level.dialog_type == 1:
            self.image.blit(self.img2, (self.img_pos))
            
        elif  self.level.dialog_type == 2:
            pass
            #self.display_lines(self.lines_game_over)
        screen.blit(self.image, (1,0))


class GameBase:
    def __init__(self):
        self.screen_tick = 0 #screen frame count
        self.screen_speed = 3 #execute every 3 frames
        self.move_tick = 0 # object motion frame count
        self.move_speed = 5 # move every 3 frames
        self.ai_enabled = False
        self.ai_speed = 10 # move every ai_speed frames
        self.ai_tick = 0 #ai motion frame count
        self.show_msg = False
        self.auto_checking = False
        self.ships_count = 0

    def game_restart(self,screen):
        pass

    def handle(self, event):
        pass
            
    def display(self, screen):
        pass
    
class BoardGame(GameBase):
    def __init__(self,mainloop,speaker,config,screen_w,screen_h,x_count,y_count):
        GameBase.__init__(self)
        self.mainloop = mainloop
        self.speaker = speaker
        self.config = config
        self.move = False # used to start moving the square when pressed
        self.direction = [0,0]
        self.ship_id = -1
        self.drag = False #used to control draging objects around
        self.gridpos_now_top = (-1,-1)
        self.gridpos_prev_top = (-1,-1)
        self.x_diff = (-1,-1)
        self.y_diff = (-1,-1)
        self.circle_lock_pos = (0,0)
        self.layout = Layout(mainloop, screen_w, screen_h,x_count,y_count)
        self.dialog = Dialog(self)
        self.screen_w=self.layout.screen_w
        self.screen_h=self.layout.screen_h
        self.mainloop.info.reset_buttons()
        self.changed_since_check = True
        self.mouse_entered_new = False
        
        #if one game has more than one category of tasks on different levels - the beginning of a category
        #can be marked on chapter list and jumped between with right click on the next level button
        self.chapters = [1]
        
        #create game board
        self.board = classes.board.Board(self.mainloop,self.layout.x_count,self.layout.y_count,self.layout.scale)
        self.create_game_objects()
        
        self.board.all_sprites_list.move_to_back(self.board.board_bg)
        self.movingsprites = pygame.sprite.RenderPlain((self.board.ship_list))
        self.staticsprites = pygame.sprite.RenderPlain((self.board.unit_list))
        self.cl_grid_line = (240, 240, 240)
        
        #to make sure game gets updated after starting
        self.mainloop.redraw_needed[0] = True
        
    def board_layout_update(self):
        self.screen_w=self.layout.screen_w
        self.screen_h=self.layout.screen_h
        self.board.scale = self.layout.scale
    
    def say(self,text,voice="1"):
        self.speaker.say(text,voice)
        
    def get_x_count(self,y_count,even=None):
        'method used to calculate the number of horizontal squares needed'
        'to fill all available area making games wider on wide screens'
        'it may make games a little bit harder on larger screens,'
        'patches my mistake to make this game optimized for 1024x786 screens only'
        scale = self.layout.avail_game_h // y_count
        x_count = self.layout.avail_game_w // scale
        if even == None:
            #if number of squares does not matter make it fill whole width rather than height
            return x_count+1
        else:
            m = x_count % 2
            if even == True and m == 0 or even == False and m == 1:
                #if the number we have is the number we need -> return it
                return x_count
            else:
                #else: increase it to make it match the criteria set (why in- and not decrease -> see: even == None)
                return x_count+1
                
    def outline_all(self,color,width,units=True,ships=True):
        #mark to draw outline around all units/ships on the board
        if units == True:
            for each in self.board.units:
                each.set_outline(color,width)
        if ships == True:
            for each in self.board.ships:
                each.set_outline(color,width)
           
    def handle(self, event):
        if event.type == pygame.MOUSEBUTTONDOWN:
            # Change the x/y screen coordinates to grid coordinates
            pos = event.pos
            column=(pos[0]-self.layout.menu_w) // (self.layout.width)
            row=pos[1] // (self.layout.height)
            
            if self.ships_count == 1:
                # if only one movable unit found - activate it
                self.board.active_ship = 0
            else:
                #check if we have clicked on any movable unit
                self.board.activate_ship(column,row)
            self.ship_id = self.board.active_ship
            self.mainloop.redraw_needed[0] = True
            if self.ship_id>=0:
                self.changed_since_check = True
                if self.board.ships[self.ship_id].draggable == True:
                    self.drag = True
                self.x_diff = column - self.board.active_ship_pos[0]
                self.y_diff = row - self.board.active_ship_pos[1]
                self.gridpos_prev_top = (column-self.x_diff,row-self.y_diff) #used to store the mouse coords on previous position in grid
                self.gridpos_now_top = (column-self.x_diff,row-self.y_diff)  #used to store current position on grid
                self.board.ships[self.board.active_ship].enable_circle() #dnable drawing circle aim on items
                self.circle_lock_pos = (self.x_diff*self.layout.scale, self.y_diff*self.layout.scale) 
                if self.board.active_val_len > 0 and self.board.ships[self.board.active_ship].readable:
                    if isinstance(self.board.ships[self.board.active_ship].value, list):                  
                        value = ', '.join(self.board.ships[self.board.active_ship].value)
                    else:
                        value = self.board.ships[self.board.active_ship].value
                    self.say(value,6)
 
        elif event.type == pygame.MOUSEMOTION:
            #make sure the current game title is displayed
            if self.mainloop.info.title != self.mainloop.m.games[self.mainloop.m.active_game_id].title:
                self.mainloop.info.reset_titles()
                #self.mainloop.info.title = self.mainloop.m.games[self.mainloop.m.active_game_id].title
                #self.mainloop.info.subtitle = self.mainloop.m.games[self.mainloop.m.active_game_id].subtitle                        
                #self.mainloop.redraw_needed[1] = True
                #print("ok", end=", ")
            if self.drag:
                pos = event.pos #pygame.mouse.get_pos()
                if pos[0] > self.layout.menu_w and pos[1] < self.layout.game_h: #if still on game board
                    self.mouse_entered_new = False
                    # Change the x/y screen coordinates to grid coordinates
                    column = (pos[0]-self.layout.menu_w) // (self.layout.width)
                    row = pos[1] // (self.layout.height)
                    
                    mdir = [0,0] #mouse drag direction
                    
                    if self.gridpos_prev_top != (column-self.x_diff, row-self.y_diff): #on_mouse_enter on a grid square simulation 
                        #mouse entered a new square
                        self.mouse_entered_new = True
                        self.mainloop.redraw_needed[0] = True
                        self.gridpos_prev_top = (column-self.x_diff, row-self.y_diff)
                        column = column - self.x_diff
                        row = row - self.y_diff

                        x_change = column - self.gridpos_now_top[0]
                        y_change = row    - self.gridpos_now_top[1]
                        
                        if -1 <= x_change <= 1 and -1 <= y_change <= 1:
                            mdir = [x_change, y_change]
                        #else if mouse is out of the range try to follow in one direction
                        else:
                            if x_change >= 1 and (self.gridpos_now_top[0] != column-self.x_diff):
                                mdir[0] = 1
                            elif x_change <= -1: #is smaller than -1
                                mdir[0] = -1
                            if y_change >= 1 and (self.gridpos_now_top[1] != column-self.y_diff):
                                mdir[1] = 1
                            elif y_change <= -1: #is smaller than -1
                                mdir[1] = -1
                            
                        if mdir[0] != 0 or mdir[1] != 0:
                            self.board.move(self.ship_id,*mdir)
                            self.board.ships[self.ship_id].turn(mdir)
                            self.gridpos_now_top = self.board.active_ship_pos
                            #self.circle_lock_pos = ((self.board.active_ship_pos[0]+x_diff)*self.layout.scale,(self.board.active_ship_pos[1]+y_diff)*self.layout.scale) 
                            self.circle_lock_pos = (self.x_diff*self.layout.scale, self.y_diff*self.layout.scale)

        elif event.type == pygame.MOUSEBUTTONUP:
            if self.drag:
                self.drag = False
                self.board.ships[self.board.active_ship].disable_circle()
                self.mainloop.redraw_needed[0] = True

        elif self.ship_id >= 0 and event.type == pygame.KEYDOWN:
            if   event.key == pygame.K_LEFT:  self.direction[0] = -1
            elif event.key == pygame.K_RIGHT: self.direction[0] =  1
            elif event.key == pygame.K_UP:    self.direction[1] = -1
            elif event.key == pygame.K_DOWN:  self.direction[1] =  1

            if self.direction[0] != 0 or self.direction[1] != 0: 
                self.move = True
                self.changed_since_check = True
                self.board.ships[self.ship_id].turn(self.direction)
                self.mainloop.redraw_needed[0] = True
                
        elif event.type == pygame.KEYUP:
            if   event.key == pygame.K_LEFT:  self.direction[0] = 0
            elif event.key == pygame.K_RIGHT: self.direction[0] = 0
            elif event.key == pygame.K_UP:    self.direction[1] = 0
            elif event.key == pygame.K_DOWN:  self.direction[1] = 0

            if self.direction == [0,0]:
                self.move = False
            else:
                self.board.ships[self.ship_id].turn(self.direction)
                self.mainloop.redraw_needed[0] = True
                
        if event.type == pygame.KEYDOWN and event.key == pygame.K_RETURN:
            if self.changed_since_check or self.show_msg == True:
                self.mainloop.redraw_needed[0] = True
                self.mainloop.redraw_needed[1] = True
                if self.show_msg == False:
                   self.check_result()
                else:
                    self.show_msg = False
                    self.level.next_board_load()
                self.changed_since_check = False
        	

    def process_keydown(self):
        self.move_tick += 1 # if key pressed execute this every move_speed frames
        if self.move_tick > self.move_speed:
            if self.move:
                self.move_tick = 0
                self.board.move(self.ship_id,*self.direction)
                self.after_keydown_move()
                self.mainloop.redraw_needed[0] = True
            
        self.screen_tick += 1 # used to limit the display update rate, without limiting responsivenes to key presses
        if self.screen_tick >= self.screen_speed:
            self.screen_tick = 0
            
    def after_keydown_move(self):
        pass
        
    def process_ai(self):
        #process ai and move unit if arrow button is pressed
        if self.show_msg == False:
            self.process_keydown()
            if self.ai_enabled == True:
                self.ai_tick += 1 # if key pressed execute this every move_speed frames
                if self.ai_tick > self.ai_speed:
                    self.ai_walk()
                    self.mainloop.redraw_needed[0] = True
                    self.ai_tick = 0

    def update(self,game):
        l = self.layout
        if self.show_msg == False:

            #Draw the board
            #self.board.draw_board(game,self.cl_grid_line,l.screen_w-l.menu_w,l.game_h,l.grid_line_w)

            #update the grid with new locations and colours
            self.board.update_ships(self.circle_lock_pos)
            
            #Draw all the spites
            self.board.all_sprites_list.draw(game)
            #self.board.sprites_to_draw.draw(game)
        else:
            self.dialog.update(game)

            
    def check_result(self):
        pass
