import pygame
import os, sys

import pygame.mixer
sounds = pygame.mixer
sounds.init()

sound_9 = '8007__cfork__cf-fx-flopp-04-bass.ogg'
s4 = sounds.Sound(os.path.join('sounds', sound_9))

class BaseButton(pygame.sprite.Sprite):
    def __init__(self,pos_x, pos_y, width, height, img_src_1="", img_src_2="", rev=False):
        pygame.sprite.Sprite.__init__(self)
        self.width = width
        self.height = height
        self.color = (70,70,70)
        self.image = pygame.Surface([width, height])
        self.image.fill(self.color)
        self.rect = self.image.get_rect()
        self.rect.topleft = [pos_x, pos_y]
        self.rect.width = width
        self.img_src_1 = img_src_1
        self.img_src_2 = img_src_2        
        points = int(round((60 * 72 /96)*1.5,0))
        self.font = pygame.font.Font(None, (points))
        self.font2 = pygame.font.Font(None, (points//2))
        self.font3 = pygame.font.Font(None, (int(points/1.5)))
        #self.font4 = pygame.font.Font(None, (points))
        self.hasimg = False
        
        if len(img_src_1)>0:
            self.load_images(rev)
        else:
            self.update_levels()

    
class Button(BaseButton):
    def __init__(self, panel, pos_x, pos_y, width, height, btntype="imgbtn", img_src_1="", img_src_2="", rev=False):
        self.btn_id = len(panel.btns)
        self.panel = panel
        self.btntype = btntype
        BaseButton.__init__(self,pos_x, pos_y, width, height, img_src_1, img_src_2, rev)
        
    def load_images(self, rev):
        self.img_pos = (0,0)
        try:
            self.img_1 = pygame.image.load(os.path.join('images', self.img_src_1)).convert()
            self.img_2 = pygame.image.load(os.path.join('images', self.img_src_2)).convert()
            #self.img_pos = (0,0)
            if rev:
                self.img_1 = pygame.transform.flip(self.img_1, 1, 0)
                self.img_2 = pygame.transform.flip(self.img_2, 1, 0)
            self.img = self.img_2
            self.hasimg = True
        except:
            pass

        self.update()
        
    def update_levels(self):
        #unsuccessful attempt to center the text
        text = self.font.render("%s" % (self.panel.level.lvl), 1, (255, 255, 255, 0))
        text2 = self.font2.render("%s/%s" % (self.panel.level.game_step, self.panel.level.games_per_lvl), 1, (255, 255, 255, 0))
        textpos1 = text.get_rect(centerx=self.image.get_width()//2)
        textpos2 = text2.get_rect(centerx=self.image.get_width()//2)
        self.image.blit(text, (textpos1[0],2))
        self.image.blit(text2, (textpos2[0],42))
        if len(self.panel.mainloop.game_board.chapters)>1:
            pygame.draw.line(self.image,((241,139,74)),[0,16],[0,48],2)
            pygame.draw.line(self.image,((241,139,74)),[4,16],[4,48],1)
            pygame.draw.line(self.image,((241,139,74)),[72,16],[72,48],2)
            pygame.draw.line(self.image,((241,139,74)),[69,16],[69,48],1)
        
    """
    def update_levels2(self):
        #unsuccessful attempt to center the text
        text = self.font.render("%s" % (self.panel.level.lvl), 1, (0, 0, 0, 0))
        text2 = self.font2.render("%s/%s" % (self.panel.level.game_step, self.panel.level.games_per_lvl), 1, (0, 0, 0, 0))
        font_x = ((54-self.font.size(str(self.panel.level.lvl))[0])//2)
        font_y = ((66-self.font.size(str(self.panel.level.lvl))[1])//3)
        self.image.blit(text, (font_x+5,font_y-7))
        font_x = ((54-self.font.size(str(self.panel.level.game_step + "/" + self.panel.level.games_per_lvl))[0])//2)
        self.image.blit(text2, (font_x+2,font_y+35))
    """ 

    def update_title(self):
        #display game titles, but only on screens with widths over 1000px
        if self.panel.mainloop.size[0] > 1000:
            text = self.font3.render("%s" % (self.panel.title), 1, (255, 255, 255, 0))
            text2 = self.font2.render("%s" % (self.panel.subtitle), 1, (255, 255, 255, 0))
            textpos1 = text.get_rect(centerx=self.image.get_width()//2)
            textpos2 = text2.get_rect(centerx=self.image.get_width()//2)
            self.image.blit(text, (0,2))
            self.image.blit(text2, (0,42))
        	#self.image.blit(text, (textpos1[0],2))
            #self.image.blit(text2, (textpos2[0],42))
    	
    def update(self):
        self.image.fill(self.color)
        if self.btntype == "imgbtn":
            self.image.blit(self.img, self.img_pos)
        elif self.btntype == "levels":
            self.update_levels()
        elif self.btntype == "titles":
        	self.update_title()

class InfoBar():
    def __init__(self,mainloop):
        self.btns = []
        self.mainloop = mainloop
        self.title = "Multiply Me"
        self.subtitle = "Multiplication Table Training"
        self.btn_list = pygame.sprite.LayeredUpdates()
        
        #self.visible_btns = [ok,left_arrow,levels,right_arrow,close,refresh,titles]
        #self.visible_btns = [1,1,1,1,1,1,1]
        self.reset_buttons()

    def new_game(self,game_board,screen):
        self.game_board = game_board
        self.level = self.game_board.level
        self.screen = screen
        self.height = self.game_board.layout.info_bar_h
        self.height_o = self.game_board.layout.info_bar_offset_h
        self.width = self.game_board.layout.game_w
        if len(self.btns) == 0:
            self.add_btns()
        self.layout_update()
        self.game_board.dialog.layout_update()

    def hover(self,pos,l):
        for btn in self.btns:
            if btn.rect.topleft[0] < (pos[0] - l.menu_w + 1) < (btn.rect.topleft[0] + btn.width) and btn.rect.topleft[1] < (pos[1] - l.info_top) < (btn.rect.topleft[1] + btn.height):
                self.mainloop.redraw_needed[1] = True                
                return btn
        return False

    def handle(self,event,layout,mainloop):
        if event.type == pygame.MOUSEBUTTONDOWN:
            # Change the x/y screen coordinates to grid coordinates
            pos = event.pos
            btn = self.hover(pos,layout)
            #if left button pressed:
            if event.button == 1:
                if btn != False:
                    if btn.btn_id == 0:
                        self.game_board.check_result()
                        btn.img = btn.img_2
                        
                    elif btn.btn_id == 1:
                        self.level.leveldown()
                        if self.level.lvl == 1:
                            btn.img = btn.img_2
    
                    elif btn.btn_id == 3:
                        self.level.levelup()
                        if self.level.lvl == self.level.lvl_count:
                            btn.img = btn.img_2
                    
                    elif btn.btn_id == 4: #clicked on close button
                        mainloop.done = True
    
                    elif btn.btn_id == 5:
                        self.level.load_level()
                        s4.play()
            #if right button pressed
            if event.button == 3:
                if btn != False:
                    if btn.btn_id == 1:
                        self.level.chapter_down()
                        if self.level.lvl == 1:
                            btn.img = btn.img_2
    
                    elif btn.btn_id == 3:
                        self.level.chapter_up()
                        if self.level.lvl == self.level.lvl_count:
                            btn.img = btn.img_2

        elif event.type == pygame.MOUSEMOTION:
            if self.mainloop.info.title != self.mainloop.m.games[self.mainloop.m.active_game_id].title:
                self.reset_titles()
            pos = event.pos
            btn = self.hover(pos,layout)
            if btn != False:
                if btn.hasimg:
                    if not ((btn.btn_id == 1 and self.level.lvl == 1) or (btn.btn_id == 3 and self.level.lvl == self.level.lvl_count) or (btn.btn_id == 0 and self.game_board.changed_since_check == False)):                 
                        btn.img = btn.img_1
            else:
                self.resetbtns()
                
    def reset_titles(self):
        self.title = self.mainloop.m.games[self.mainloop.m.active_game_id].title
        self.subtitle = self.mainloop.m.games[self.mainloop.m.active_game_id].subtitle                        
        self.mainloop.redraw_needed[1] = True
        self.mainloop.m.mouseenter = -1
        self.mainloop.m.mouseenter_cat = -1
        
    def resetbtns(self):
        for btn in self.btns:
            if btn.hasimg:
                btn.img = btn.img_2
        
    def add_btn(self, panel, pos_x, pos_y, btn_size_x, btn_size_y, btntype="imgbtn", img_src_1="", img_src_2="", rev=False):
        new_button = Button(panel, pos_x, pos_y, btn_size_x, btn_size_y, btntype, img_src_1, img_src_2, rev)
        self.btns.append(new_button)
        self.btn_list.add(new_button)

    def add_btns(self):
        self.add_btn(self,5,5,84,66,"imgbtn","ok.png", "ok_2.png")
        self.add_btn(self,self.width-303,5,64,66,"imgbtn","arrow.png", "arrow_2.png")
        self.add_btn(self,self.width-233,5,74,66,"levels")#level number label
        self.add_btn(self,self.width-153,5,64,66,"imgbtn","arrow.png","arrow_2.png",True)
        self.add_btn(self,self.width-71,5,66,66,"imgbtn","close.png","close_2.png")
        self.add_btn(self,105,5,63,66,"imgbtn","refresh.png","refresh_2.png")
        title_width = self.width-303 - (168+5+20)-5
        self.add_btn(self,168+5+10,5,title_width,66,"titles")
        self.btn_list.move_to_back(self.btns[6]) 


    def layout_update(self):
        
        self.btns[1].rect.left = self.width-303
        self.btns[2].rect.left = self.width-233
        self.btns[3].rect.left = self.width-153
        self.btns[4].rect.left = self.width-71   
        #self.btns[6].rect.width = 100        
        self.reset_alignment()
        self.check_btn_tops()

    def align_to_left(self):
        if self.visible_btns[0] == 0 and self.visible_btns[5] == 1:
            #self.btns[0].rect.left = 5
            self.btns[5].rect.left = 5
            self.btns[6].rect.left = 78
        elif self.visible_btns[0] == 0 and self.visible_btns[5] == 0:
            self.btns[6].rect.left = 5
        elif self.visible_btns[0] == 1 and self.visible_btns[5] == 0:
            self.btns[0].rect.left = 5
            self.btns[6].rect.left = 105
        
    def reset_alignment(self):
        self.btns[0].rect.left = 5
        self.btns[5].rect.left = 105
        self.btns[6].rect.left = 183
            
    def hide_buttons(self,a,b,c,d,e,f,g):
        self.visible_btns = [a,b,c,d,e,f,g]
        
    def reset_buttons(self):
        self.visible_btns = [1,1,1,1,1,1,1]
        
    def check_btn_tops(self):
        #self.visible_btns = [ok,left_arrow,levels,right_arrow,close,refresh,titles]
        #self.visible_btns = [1,1,1,1,1,1,1]
        #if sum(self.visible_btns) < 7:
        for i in range(7):
            if self.visible_btns[i]==0:
                self.btns[i].rect.top = -200
            else:
                self.btns[i].rect.top = 5
        if self.visible_btns[0] == 0 or self.visible_btns[5] == 0:
            self.align_to_left()
        
    def draw(self,screen,screen2):
        #draw info bar
        screen.fill((70,70,70))
        screen2.fill((70,70,70))
        pygame.draw.line(screen2,((255,255,255)),[0,0],[0,self.height_o],1)
        
        
        for each_item in self.btns:
            each_item.update()

        self.btn_list.draw(screen)
        