import pygame
import classes.game_driver as gd
from game_boards import *
import classes.colors
import os, sys

import pygame.mixer
sounds = pygame.mixer
sounds.init()

sound_7 = '8005__cfork__cf-fx-flopp-02-juicy.ogg'
sound_9 = '8007__cfork__cf-fx-flopp-04-bass.ogg'
s3 = sounds.Sound(os.path.join('sounds', sound_7))
s4 = sounds.Sound(os.path.join('sounds', sound_9))

cl = classes.colors.Color()

class MenuCategory(pygame.sprite.Sprite):
    def __init__(self, cat_id, title, subtitle, cat_icon_size,img_src):
        # Call the parent's constructor
        pygame.sprite.Sprite.__init__(self)
        self.cat_id = cat_id
        self.title = title
        self.subtitle = subtitle
        self.color = (255, 0, 255)

        self.image = pygame.Surface([cat_icon_size, cat_icon_size])
        self.image.fill(self.color)
        self.img_src = img_src
        if len(self.img_src) > 0:
            self.img = self.image
            self.img_pos = (0,0)
            try:
                self.img_org = pygame.image.load(os.path.join('icons', self.img_src)).convert()
                self.img_pos = (0,0)
                self.img = self.img_org
            except:
                pass

        self.image.set_colorkey(self.color)
        
        # Make our top-left corner the passed-in location. The +1 is the margin
        self.rect = self.image.get_rect()
        
    def update(self):
        self.image.fill(self.color)
        if len(self.img_src) > 0:
            self.image.blit(self.img, self.img_pos)
            
class MenuItem(pygame.sprite.Sprite):
    def __init__(self, item_id, cat_id, title, subtitle, constructor, icon_size, img_src):
        # Call the parent's constructor
        pygame.sprite.Sprite.__init__(self)
        self.item_id = item_id
        self.cat_id = cat_id
        self.game_constructor = constructor
        self.title = title
        self.subtitle = subtitle
        self.color = (255, 0, 255)

        self.image = pygame.Surface([icon_size, icon_size])
        self.img_src = img_src
        if len(self.img_src) > 0:
            self.img = self.image
            self.img_pos = (0,0)
            try:
                self.img_org = pygame.image.load(os.path.join('icons', self.img_src)).convert()
                self.img_pos = (0,0)
                self.img = self.img_org
            except:
                pass

        self.image.set_colorkey(self.color)
 
        # Make our top-left corner the passed-in location. The +1 is the margin
        self.rect = self.image.get_rect()

    def update(self):
        self.image.fill(self.color)
        if len(self.img_src) > 0:
            self.image.blit(self.img, self.img_pos)

class MenuBookmark(pygame.sprite.Sprite):
    def __init__(self, bm_id, bm_icon_size, img_src):
        # Call the parent's constructor
        pygame.sprite.Sprite.__init__(self)
        self.bm_id = bm_id
        self.color = (255, 0, 255)

        self.image = pygame.Surface([61, 63])
        self.image.fill(self.color)
        self.img_src = img_src
        if len(self.img_src) > 0:
            self.img = self.image
            self.img_pos = (0,0)
            try:
                self.img_org = pygame.image.load(os.path.join('icons', self.img_src)).convert()
                self.img_pos = (0,0)
                self.img = self.img_org
            except:
                pass

        self.image.set_colorkey(self.color)
        
        # Make our top-left corner the passed-in location. The +1 is the margin
        self.rect = self.image.get_rect()

    def update(self):
        self.image.fill(self.color)
        if len(self.img_src) > 0:
            self.image.blit(self.img, self.img_pos)

class MenuScrollBtn(pygame.sprite.Sprite):
    def __init__(self, sb_id, sb_icon_size, img_src):
        # Call the parent's constructor
        pygame.sprite.Sprite.__init__(self)
        self.sb_id = sb_id
        self.color = (255, 255, 255)
        #, pygame.SRCALPHA
        self.image = pygame.Surface(sb_icon_size)
        #self.image.set_alpha(100)
        self.image.fill(self.color)
        self.img_src = img_src
        if len(self.img_src) > 0:
            self.img = self.image
            self.img_pos = (0,0)
            try:
                self.img_org = pygame.image.load(os.path.join('icons', self.img_src)).convert()
                self.img_pos = (0,0)
                self.img = self.img_org
            except:
                pass
        #self.image.convert_alpha()
        self.image.set_colorkey(self.color)
        
        
        # Make our top-left corner the passed-in location. The +1 is the margin
        self.rect = self.image.get_rect()

    def update(self):
        #self.image.fill(self.color)
        if len(self.img_src) > 0:
            self.image.blit(self.img, self.img_pos)
    
class Menu:
    def __init__(self,mainloop):
        self.categories = []
        self.games = []
        self.games_current = []
        self.bookmarks = []
        self.scroll_btns = []
        self.mouseenter = -1
        self.mouseenter_cat = -1
        self.mainloop = mainloop
        self.l = None
        self.active_game_id = 0
        self.game_started_id = -1
        self.active_cat = 0
        self.tab_game_id = 0
        self.prev_cat = -1
        self.game_constructor = game000.Board
        self.icon_size = 50
        self.cat_icon_size = 50
        self.x_margin = 6 #6
        self.y_margin = 7 #5
        
        self.scroll_l = 0
        self.scroll_r = 0
        self.tab_l_scroll = 0
        self.tab_r_scroll = 0
        
        self.scroll_step = self.cat_icon_size + self.y_margin
        
        # This is a list of 'sprites.' Each block in the program is
        # added to this list. The list is managed by a class called 'RenderPlain.'
        self.categories_list = pygame.sprite.LayeredUpdates()
        self.games_in_current_cat = pygame.sprite.LayeredUpdates()
        self.bookmarks_list = pygame.sprite.LayeredUpdates()
        #self.scroll_btns_list = pygame.sprite.LayeredUpdates()
        #self.menu_l_list = pygame.sprite.LayeredUpdates()
        #self.menu_r_list = pygame.sprite.LayeredUpdates()
        self.add_categories()
        self.add_games()
        self.add_bookmark("","tab_l.png")
        self.categories_list.add(self.bookmarks[0])
        self.categories_list.move_to_back(self.bookmarks[0])
        self.add_bookmark("","tab_r.png")
        self.add_scroll_btns()
        self.update_scroll_pos()
        self.categories_list.add(self.scroll_btns[0])
        self.categories_list.move_to_front(self.scroll_btns[0])
        
    def handle_menu_l(self,event):
        try:
            if event.type == pygame.MOUSEBUTTONDOWN:
                pos = event.pos
                #if y is within category size
                if self.x_margin < pos[0] < self.cat_icon_size + self.x_margin:
                    if self.y_margin+self.l.misio_pos[3] < pos[1] < self.cat_h+self.l.misio_pos[3]+self.scroll_l:
                        row = (pos[1]-3-self.l.misio_pos[3]-self.scroll_l) // (self.cat_icon_size + self.y_margin)
                        self.active_cat = row
                        self.tab_l_scroll = (self.scroll_l // self.scroll_step)
                        s3.play()
                        self.mainloop.redraw_needed[2] = True
            elif event.type == pygame.MOUSEMOTION:
                pos = event.pos
                if self.x_margin < pos[0] < self.cat_icon_size + self.x_margin:
                    if self.y_margin+self.l.misio_pos[3] < pos[1] < self.cat_h+self.l.misio_pos[3]:
                        row = (pos[1]-3-self.l.misio_pos[3]-self.scroll_l) // (self.cat_icon_size + self.y_margin)
                        if row != self.mouseenter_cat:
                            self.mouseenter_cat = row
                            self.mouseenter = -1
                            self.mainloop.info.title = self.categories[row].title
                            self.mainloop.info.subtitle = self.categories[row].subtitle
                            self.mainloop.redraw_needed[1] = True
                    else:
                        self.reset_titles()
                        
                    if pos[1] > self.mainloop.size[1]-20 and self.scroll_l == 0:
                        menu_height = self.mainloop.size[1]-(self.y_margin+self.l.misio_pos[3])
                        if self.cat_h > menu_height:
                            diff = self.cat_h - menu_height
                            while -self.scroll_l < diff:
                                self.scroll_l -= self.scroll_step
                            self.tab_l_scroll = (self.scroll_l // self.scroll_step)
                            self.mainloop.redraw_needed[2] = True
                            #self.mainloop.info.title = "entered the scroll zone"
                            
                    elif self.l.misio_pos[3]-20 < pos[1] < self.l.misio_pos[3]+5 and self.scroll_l < 0:
                        self.scroll_l = 0
                        self.tab_l_scroll = 0 #(self.scroll_r // self.scroll_step)
                        self.mainloop.redraw_needed[2] = True
                        #self.mainloop.info.title = "entered the scroll zone"
                else:
                    self.reset_titles()
        except:
            pass
            
    def handle_menu_r(self,event,mlw):
        try:
            if event.type == pygame.MOUSEBUTTONDOWN:
                pos = event.pos
                if (mlw + self.x_margin) < pos[0] < (mlw + self.icon_size + self.x_margin):
                    if self.y_margin+self.l.misio_pos[3] < pos[1] < self.game_h+self.l.misio_pos[3]:
                        row = (pos[1]-3-self.l.misio_pos[3]-self.scroll_r) // (self.icon_size + self.y_margin)
                        self.active_game_id = self.games_current[row].item_id
                        self.tab_r_scroll = (self.scroll_r // self.scroll_step)
                        self.tab_game_id = row
                        self.game_constructor = self.games_current[row].game_constructor
                        s4.play()
                        self.mainloop.score = 0  
                        self.mainloop.redraw_needed = [True, True, True]
            elif event.type == pygame.MOUSEMOTION:
                pos = event.pos
                if (mlw + self.x_margin) < pos[0] < (mlw + self.icon_size + self.x_margin):
                    if self.y_margin+self.l.misio_pos[3] < pos[1] < self.game_h+self.l.misio_pos[3]:
                        row = (pos[1]-3-self.l.misio_pos[3]-self.scroll_r) // (self.icon_size + self.y_margin)
                        if row != self.mouseenter:
                            self.mouseenter = row
                            self.mouseenter_cat = -1
                            self.mainloop.info.title = self.games_current[row].title
                            self.mainloop.info.subtitle = self.games_current[row].subtitle
                            self.mainloop.redraw_needed[1] = True
                    else:
                        self.reset_titles()
                    if pos[1] > self.mainloop.size[1]-20 and self.scroll_r == 0:
                        menu_height = self.mainloop.size[1]-(self.y_margin+self.l.misio_pos[3])
                        if self.game_h > menu_height:
                            diff = self.game_h - menu_height
                            while -self.scroll_r < diff:
                                self.scroll_r -= self.scroll_step
                            self.tab_r_scroll = (self.scroll_r // self.scroll_step)
                            self.mainloop.redraw_needed[2] = True
                            #self.mainloop.info.title = "entered the scroll zone"
                    elif self.l.misio_pos[3]-20 < pos[1] < self.l.misio_pos[3]+5 and self.scroll_r < 0:
                        self.scroll_r = 0
                        self.tab_r_scroll = 0 #(self.scroll_r // self.scroll_step)
                        self.mainloop.redraw_needed[2] = True
                        #self.mainloop.info.title = "entered the scroll zone"
                else:
                    self.reset_titles()
        except:
            pass
            
    def reset_titles(self):
        self.mainloop.info.title = ""
        self.mainloop.info.subtitle = ""
        self.mainloop.redraw_needed[1] = True
        self.mouseenter = -1
        self.mouseenter_cat = -1
        
    def add_bookmark(self,title,img_src):
        new_bookmark = MenuBookmark(len(self.bookmarks),self.cat_icon_size,img_src)
        self.bookmarks.append(new_bookmark)

    def add_scroll_btn(self,img_src):
        new_scroll_btn = MenuScrollBtn(len(self.scroll_btns),[55,13],img_src)
        self.scroll_btns.append(new_scroll_btn)
        #self.scroll_btns_list.add(new_scroll_btn)

    def add_scroll_btns(self):
        self.add_scroll_btn("arrow_l2.png")
        self.add_scroll_btn("arrow_r2.png")
            
    def update_scroll_pos(self):
        pos = [[9+self.x_margin,self.mainloop.size[1]-13],[9+self.x_margin,self.mainloop.size[1]-13]]
        i=0        
        for each_item in self.scroll_btns:
            each_item.rect.topleft = pos[i]
            i+=1
        
    def add_category(self,title,subtitle,img_src):
        new_category = MenuCategory(len(self.categories),title,subtitle,self.cat_icon_size,img_src)
        self.categories.append(new_category)
        self.categories_list.add(new_category)
        #self.menu_l_list.add(new_category)

    def add_categories(self):
        #/home/tiger/Documents/aapython/MyPrograms/board/icons/c_ico_geometry.png
        self.add_category("Keyboard & Mouse","","c_ico_mkbrdi.png")
        self.add_category("Letters and Words","","c_ico_abc.png")
        self.add_category("Arithmetic","Numbers & Basic Operations","c_ico_operators.png")
        self.add_category("Arithmetic ","Sorting and Comparing","c_ico_math2.png")
        self.add_category("Shapes","","c_ico_geometry.png")
        self.add_category("Art","","c_ico_art.png")
        self.add_category("Memory","","c_ico_memory.png")
        self.add_category("Games & Mazes","","c_ico_games.png")
        #self.add_category("Under Construction","","ico_cone.png")

        self.cat_h = len(self.categories)*(self.cat_icon_size+self.y_margin)#-self.y_margin

    def add_game(self,cat_id,constructor,title,subtitle,img_src):
        new_game = MenuItem(len(self.games),cat_id,title,subtitle,constructor,self.icon_size,img_src)
        self.games.append(new_game)

    def add_games(self):
        #self.add_game(20,"title",game000.Board,"ico_apple1.png")#intro
        self.add_game(0,game000.Board,"About.","Game info and credits...","ico_info.png")
        """        
        self.add_game(0,game000.Board,"Mouse","","ico_mouse_g.png")
        self.add_game(0,game000.Board,"Keyboard","","ico_kbrd_g.png")
        self.add_game(0,game000.Board,"Fish","","ico_fish.png")
        self.add_game(0,game000.Board,"Window Cleaning","","ico_squeegee.png")
        """
        self.add_game(0,game030.Board,"Hit the Mole","","ico_mole.png")

        self.add_game(1,game037.Board,"Letters","Learn Letters with Flashcards","ico_handwriting.png")
        self.add_game(1,game010.Board,"Learn to Write","Trace Letters and Numbers","ico_tablet.png")#colours
        self.add_game(1,game014.Board,"Complete the ABC","","ico_abc2.png") #recreate abc
        self.add_game(1,game006.Board,"Sorting Letters","Lowercase Letters","ico_sort_lc.png")#sorting lowercase letters
        self.add_game(1,game007.Board,"Sorting Letters ","Uppercase Letters","ico_sort_uc.png")#sorting uppercase letters
        self.add_game(1,game013.Board,"Word Builder","","ico_words.png") #build a word
        self.add_game(1,game023.Board,"Word Maze","Collect all letters in the right order","ico_bug.png")#letter labyrinth
        self.add_game(1,game025.Board,"Word Maze + 4","Collect all letters in the right order","ico_bug2.png")#letter labyrinth with AI

        self.add_game(2,game038.Board,"Numbers","Learn Numbers with Flashcards","ico_flashcards.png")
        self.add_game(2,game001.Board,"Learn to Count","Basic Addition & Subtraction","ico_apple1.png")#apples
        self.add_game(2,game027.Board,"Shopping List","","ico_basket.png")#Shop
        self.add_game(2,game036.Board,"Plus or Minus","","ico_plusminus.png")#operation slider 2 [+,-] 2 = 4
        self.add_game(2,game039.Board,"Basic Operations","","ico_operations.png")#+,-,x,:
        self.add_game(2,game019.Board,"Basic Operations ","Reversed","ico_operations2.png")#+,-,x,:
        self.add_game(2,game004.Board,"Multiplication Table","Find the product","ico_mul1.png")#multiplication table 2 x 2 = ?
        self.add_game(2,game034.Board,"Multiplication Table ","Find the multiplier","ico_mul2.png")#multiplication table ? x 2 = 4
        self.add_game(2,game035.Board,"Division","","ico_div.png")#multiplication table 4 : 2 = ?

        self.add_game(3,game005.Board,"Sorting Numbers","","ico_sort.png")#sorting numbers
        self.add_game(3,game032.Board,"Number Comparison","","ico_sign1.png")#lesser, greater or equal - with slider
        self.add_game(3,game033.Board,"Addition & Subtraction","Comparison","ico_sign2.png")#lesser, greater or equal +/- with slider
        self.add_game(3,game026.Board,"Fractions","Comparison","ico_frac.png")#lesser, greater or equal - fractions
        self.add_game(3,game020.Board,"Decimal Fractions","Comparison","ico_frac2.png")#lesser, greater or equal - fractions        
        self.add_game(3,game011.Board,"Even or Odd","","ico_odd.png")#even, odd numbers

        self.add_game(4,game009.Board,"Shapes","Learn Shapes wih Flashcards","ico_geometry.png")
        self.add_game(4,game008.Board,"Shape Matching","help me find my shadow","ico_duck.png")
        
        self.add_game(5,game021.Board,"Paint","","ico_paint.png")
        self.add_game(5,game042.Board,"Colour Matching","label the colours","ico_colors.png")
        
        self.add_game(6,game012.Board,"Follow the Arrows","remember the directions","ico_owl.png")
        self.add_game(6,game002.Board,"Photographic Memory","Training","ico_memory2.png")
        self.add_game(6,game003.Board,"Photographic Memory","Automatic Levels","ico_memory3.png")
        
        self.add_game(7,game029.Board,"Mouse Maze","Let's have some cheese","ico_mouse.png")#mouse random 
        self.add_game(7,game028.Board,"Sheep Maze","Find the rest of the herd","ico_sheep2.png") #laby
        self.add_game(7,game040.Board,"Connect","Numbers","ico_balloonsx.png")#baloons 2
        self.add_game(7,game041.Board,"Connect ","Balloons with threads","ico_balloons.png")#baloons 2
        self.add_game(7,game015.Board,"Fifteen","With a Twist","ico_15a.png")#Fifteen
        
        """
        self.add_game(8,game031.Board,"title30","","ico_fish.png")
        self.add_game(8,game016.Board,"title32","","ico_unknown.png")
        self.add_game(8,game017.Board,"title33","","ico_unknown.png")
        self.add_game(8,game018.Board,"title34","","ico_unknown.png")
        self.add_game(8,game022.Board,"title35","","ico_mouse.png")#labyrinth builder
        self.add_game(8,game024.Board,"title36","","ico_bug.png")#labyrinth builder - 1/4
        """

    def draw_menu(self,menu,menu_l,menu_r,l):
        mw = l.menu_r_w
        menu.fill((70,70,70))
        menu_l.fill(cl.menu_l)
        menu_r.fill(cl.menu_r)
        pygame.draw.line(menu_l,cl.black,[l.menu_l_w-1,0],[l.menu_l_w-1,l.screen_h],1)
        pygame.draw.line(menu_r,cl.black,[l.menu_r_w-1,0],[l.menu_r_w-1,l.screen_h],1)

        #load games from new category if changed
        self.change_category(self.active_cat)

        x=self.x_margin
        y=self.y_margin+l.misio_pos[3]+self.scroll_l
        for each_item in self.categories:
            each_item.rect.topleft = [x, y]
            y += self.icon_size + self.y_margin
            each_item.update()
            
        x=self.x_margin
        y=self.y_margin+l.misio_pos[3]+self.scroll_r
        c=5
        for each_item in self.games_current:
            each_item.rect.topleft = [x, y]
            each_item.update()
            y += self.cat_icon_size + self.y_margin
            c += 10
        #if category with current game is shown show the tab, otherwise hide it (move it off screen) 
        if self.games[self.active_game_id] in self.games_in_current_cat:
            bmr_top = (self.tab_game_id+self.tab_r_scroll)*(self.icon_size+self.y_margin)+2+l.misio_pos[3]
        else:
            bmr_top = -100
        bml_top = (self.active_cat+self.tab_l_scroll)*(self.icon_size+self.y_margin)+2+l.misio_pos[3]
        self.bookmarks[0].rect.topleft = [2,bml_top-1]
        self.bookmarks[1].rect.topleft = [2,bmr_top-1]
        
        self.bookmarks[0].update()
        self.bookmarks[1].update()
        
       
        #Draw all the spites
        self.categories_list.draw(menu_l)        
        #self.scroll_btns_list.draw(menu_l)
        self.games_in_current_cat.draw(menu_r)
        #self.bookmarks_list.draw(menu_l)
        for each in self.scroll_btns:
            each.update()
                    
    def change_category(self, cat_id):
        if self.prev_cat != self.active_cat:
            self.scroll_r = 0
            #self.tab_game_id += self.tab_r_scroll
            self.tab_r_scroll = 0
            self.games_in_current_cat.empty()
            self.games_current = []
            for each_item in self.games:
                if each_item.cat_id == cat_id:
                    self.games_in_current_cat.add(each_item)
                    self.games_current.append(each_item)
            self.games_in_current_cat.add(self.bookmarks[1])
            self.games_in_current_cat.move_to_back(self.bookmarks[1])
            self.prev_cat = self.active_cat
            self.games_in_current_cat.add(self.scroll_btns[1])
            self.games_in_current_cat.move_to_front(self.scroll_btns[1])
            self.game_h = len(self.games_current)*(self.icon_size+self.y_margin)#-self.y_margin
            
            