import classes.level_controller as lc
import classes.game_driver as gd
import classes.extras as ex

import classes.board
import random
import colorsys

class Board(gd.BoardGame):
    def __init__(self, mainloop, speaker, config, screen_w, screen_h):
        self.level = lc.Level(self,99,9)
        gd.BoardGame.__init__(self,mainloop,speaker,config,screen_w,screen_h,12,5)
        
        
    def create_game_objects(self, level = 1):
        #create non-movable objects
        s = random.randrange(150, 190, 5)
        v = random.randrange(230, 255, 5)
        h = random.randrange(0, 255, 5)
        
        color0 = ex.hsv_to_rgb(h,40,230) #highlight 1
        color1 = ex.hsv_to_rgb(h,70,v) #highlight 2
        color2 = ex.hsv_to_rgb(h,s,v) #normal color
        color3 = ex.hsv_to_rgb(h,230,100)
        white = [255,255,255]

        #data = [x_count, y_count, number_count, top_limit, ordered]
        if self.level.lvl == 1:
            data = [12,5,10,10,1]
        elif self.level.lvl == 2:
            data = [12,5,10,20,2]
        elif self.level.lvl == 3:
            data = [12,5,10,99,2]
        elif self.level.lvl == 4:
            data = [12,5,15,20,2]
        elif self.level.lvl == 5:
            data = [12,5,15,50,2]
        elif self.level.lvl == 6:
            data = [12,5,15,99,2]
        elif self.level.lvl == 7:
            data = [12,5,20,30,2]
        elif self.level.lvl == 8:
            data = [12,5,20,50,2]
        elif self.level.lvl == 9:
            data = [12,5,20,99,2]
        self.chapters = [1,4,7,9]

        self.data = data
        self.layout.update_layout(data[0],data[1])
        self.board.level_start(data[0],data[1],self.layout.scale)
        
        self.num_list = []
        
        choice_list = [x for x in range(1,data[3]+1)]
        for i in range(data[2]):
            index = random.randrange(0,len(choice_list))
            self.num_list.append(choice_list[index])
            del(choice_list[index])

        color = ((255,255,255))
        self.board.add_unit(0,0,2,2,classes.board.Label,"Even",color2,"",1)
        self.board.add_unit(0,2,2,2,classes.board.Label,"Odd",color2,"",1)
        
        self.board.add_door(2,0,data[0]-2,2,classes.board.Door,"",white,"")
        self.board.units[2].door_outline = True

        
        #find position of first door square
        x = data[0]-1 #(data[0]-data[2])//2
        y = data[1]-2
        #add objects to the board
        for i in range(data[2]):
            h = random.randrange(0, 255, 5)
            number_color = ex.hsv_to_rgb(h,s,v) #highlight 1
            caption = str(self.num_list[i])
            self.board.add_unit(x,y,1,1,classes.board.Letter,caption,number_color,"",data[4])
            x -= 1            
            if x <= 1:
                x =  data[0]-1
                y -= 1
        
        instruction = "Find and separate the Even Numbers form the Odd Numbers in the above series."
        self.board.add_unit(0,data[1]-1,data[0],1,classes.board.Label,instruction,color0,"",7)
        self.outline_all(0,1)
        self.board.all_sprites_list.move_to_front(self.board.units[2]) 
        if self.level.lvl == 1 and self.level.game_step == 1 and self.config.read_inst:        
            self.say(instruction)

    def handle(self,event):
        gd.BoardGame.handle(self, event) #send event handling up

    def update(self,game):
        game.fill((255,255,255))
        gd.BoardGame.update(self, game) #rest of painting done by parent

    def check_result(self):
        if self.changed_since_check:
            correct = True
            for each in self.board.ships:
                if each.grid_y < 2 and self.num_list[each.unit_id]%2 != 0 \
                or each.grid_y > 1 and self.num_list[each.unit_id]%2 == 0:
                    correct = False
            if correct == True:
                self.level.next_board()
            else:
                self.say("Sorry! It is wrong.",6)
                self.changed_since_check = False
                