import classes.level_controller as lc
import classes.game_driver as gd
import classes.extras as ex

import classes.board
import random
import pygame
#import colorsys


class Board(gd.BoardGame):
    def __init__(self, mainloop, speaker, config,  screen_w, screen_h):
        self.level = lc.Level(self,1,10)
        gd.BoardGame.__init__(self,mainloop,speaker,config,screen_w,screen_h,11,9)
        
        
    def create_game_objects(self, level = 1):
        self.ai_enabled = True
        self.board.draw_grid = False
        #create non-movable objects
        #self.move_speed = 10
        s = random.randrange(30, 80)
        v = random.randrange(200, 255)
        h = random.randrange(0, 255)
        color = ex.hsv_to_rgb(h,s,v)
        white = [255,255,255]
        
        if self.level.lvl == 1:
            data = [7,5,17,-2]
        elif self.level.lvl == 2:
            data = [7,5,17,-1]
        elif self.level.lvl == 3:
            data = [7,5,14,-2]
        elif self.level.lvl == 4:
            data = [7,5,14,-1]
        elif self.level.lvl == 5:
            data = [7,5,12,-2]
        elif self.level.lvl == 6:
            data = [7,5,12,-1]
        elif self.level.lvl == 7:
            data = [7,5,10,-2]
        elif self.level.lvl == 8:
            data = [7,5,10,-1]
        elif self.level.lvl == 9:
            data = [7,5,8,-2]
        elif self.level.lvl == 10:
            data = [7,5,8,-1]
            
        self.ai_speed = data[2]
        #stretch width to fit the screen size
        max_x_count = self.get_x_count(data[1],even=False)
        if max_x_count > 7:
            data[0] = max_x_count
            
        self.data = data
        self.level.game_step = 0
        self.level.games_per_lvl = 1
        self.moveable = False
        self.moves = []
        self.move_buttons = []
        self.possible_move_buttons = []
        self.sequence_counter = 0
        self.current_step = 0
        self.start_sequence = True
        self.completed_mode = False
        
        self.center = [data[0]//2,data[1]//2]
        self.mainloop.info.hide_buttons(0,1,1,1,1,1,1)
        
        self.layout.update_layout(data[0],data[1])
        scale = self.layout.scale
        self.board.level_start(data[0],data[1],scale)
        
        #self.board.add_unit(self.center[0],self.center[1],1,1,classes.board.ImgShip,"",white,"owl_150.png")
        self.board.add_unit(self.center[0],self.center[1],1,1,classes.board.MultiImgSprite,"",white,"owl_5.png",0,frame_flow = [0,1,2,3,4,3,2,1,0],frame_count=9)
        self.owl = self.board.ships[0]
        self.owl.outline = False
        self.owl.draggable = False
        
        self.board.active_ship = 0
        self.ship_id = 0
        self.images = ["a_yellow_150.png","a_green_150.png","a_blue_150.png","a_red_150.png"]
        #self.images2 = ["a_yellow2_150.png","a_green2_150.png","a_blue2_150.png","a_red2_150.png"]
        for i in range(4):
            self.board.add_door(self.center[0],self.center[1],1,1,classes.board.SlidingDoor,"",white,self.images[i],frame_flow=[0,1], frame_count=2)

        #self.board.add_door(center[0]+1,center[1],1,1,classes.board.Door,"",white,"a_green_150.png")
        #self.board.add_door(center[0],center[1]-1,1,1,classes.board.Door,"",white,"a_blue_150.png")
        #self.board.add_door(center[0],center[1]+1,1,1,classes.board.Door,"",white,"a_red_150.png")

        self.update_arrows()
        self.board.all_sprites_list.move_to_front(self.board.ships[0])
        self.add_next_move()

        #self.outline_all(1,1)
        
    def handle(self,event):
        gd.BoardGame.handle(self, event) #send event handling up
        if self.ship_id < 0 and event.type == pygame.MOUSEBUTTONDOWN:
            #make it impossible to deselect the main character
            self.board.active_ship = 0
            self.ship_id = 0
        if event.type == pygame.KEYDOWN and self.moveable == False:
            self.move = False
            """
            btn_id = -1
            if   event.key == pygame.K_LEFT:  btn_id = 0
            elif event.key == pygame.K_RIGHT: btn_id = 1
            elif event.key == pygame.K_UP:    btn_id = 2
            elif event.key == pygame.K_DOWN:  btn_id = 3
            
            #if btn_id > -1:
                #self.highlight_color(btn_id)
            """
        if event.type == pygame.KEYUP:
             self.highlight_color(-1)
        self.mainloop.redraw_needed[0] = True
        
    def update_arrows(self):
        directions = [[-1,0],[1,0],[0,-1],[0,1]]
        self.owl_pos = list(self.board.active_ship_pos)
        self.possible_moves = []
        self.possible_move_buttons = []
        for i in range(4):
            if 0 <= self.owl_pos[0]+directions[i][0] < self.data[0] and 0 <= self.owl_pos[1]+directions[i][1] < self.data[1]:
                pos = [self.owl_pos[0]+directions[i][0],self.owl_pos[1]+directions[i][1]]
                self.possible_moves.append(pos)
                self.possible_move_buttons.append(i)
            else:
                pos = self.owl_pos
            self.board.units[i].set_pos(pos)
        #print(self.possible_moves)
        self.mainloop.redraw_needed[0] = True
        
    def update(self,game):
        game.fill((255,255,255))
        gd.BoardGame.update(self, game) #rest of painting done by parent

    def after_keydown_move(self):
        
        self.update_arrows()
        if self.owl_pos == self.moves[self.current_step]:
            self.highlight_color(self.move_buttons[self.current_step])
            if self.current_step < len(self.moves)-1:
                self.current_step += 1
                self.level.game_step = self.current_step
            else:
                self.level.game_step = self.current_step + 1
                self.ai_speed = 5
                self.completed_mode = True
                self.ai_enabled = True
                #self.highlight_color(self.move_buttons[self.current_step-1])
            self.mainloop.redraw_needed[1] = True
            self.mainloop.redraw_needed[0] = True
        else:
            #print("owl_pos: {0}, moves: {1}, current_step: {2}".format(self.owl_pos,self.moves[self.current_step],self.current_step))
            self.game_over()
        self.move = False
        
                
    
    #def possible_moves(self):
    #    current_pos = self.owl_pos
        
    def next_level(self):
        self.current_step = 0
        self.board._place_unit(0, self.center)
        self.update_arrows()
    
    def game_over(self):
        #self.level.lvl = 1
        self.level.games_per_lvl = 1
        self.level.game_step = 0
        self.mainloop.redraw_needed[1] = True
        self.level.game_over()
        
        
    def highlight_color(self,btn_id):
        for i in range(4):
            if i == btn_id:
                self.board.units[i].set_frame(1)
            else:
                self.board.units[i].set_frame(0)
        
    def add_next_move(self):
        next, btn = self.pick_index()
        if len(self.moves) > -1 - self.data[3]:
            while btn == self.move_buttons[-1] and btn == self.move_buttons[self.data[3]]:
                next, btn = self.pick_index()
        """
        index = random.choice(range(len(self.possible_moves)))
        next = self.possible_moves[index]
        btn = self.possible_move_buttons[index]
        if len(self.moves) > 1:
            if btn==self.move_buttons[-1]==self.move_buttons[-2]:"""
        self.moves.append(next) #possible_moves = self.possible_moves()
        self.move_buttons.append(btn)
        #print(self.move_buttons)

    def pick_index(self):
        index = random.choice(range(len(self.possible_moves)))
        next = self.possible_moves[index]
        btn = self.possible_move_buttons[index]
        return [next, btn]    

    def ai_walk(self):
        if self.start_sequence:
            if self.sequence_counter < len(self.moves)*2:
                if self.sequence_counter % 2 == 0:
                    self.highlight_color(self.move_buttons[self.sequence_counter//2])
                else:
                    self.highlight_color(-1)
                self.sequence_counter += 1
            else:
                self.start_sequence = False
                self.ai_enabled = False
                self.sequence_counter = 0
                self.moveable = True
        elif self.completed_mode:
            if self.owl.frame < self.owl.frame_count - 1:
                self.owl.next_frame()
            else:
                self.check_result()
        
    def check_result(self):
        if self.current_step == len(self.moves)-1:
            self.add_next_move()
            self.next_level()
            self.level.games_per_lvl = len(self.moves) #gpl #number of games to play in order to level up
            #self.level.lvl_count += 1
            #self.level.lvl += 1 
            self.level.game_step = 0
            self.owl.set_frame(0)
            self.mainloop.redraw_needed[1] = True            
            self.completed_mode = False
            self.start_sequence = True
            self.ai_enabled = True
            self.ai_speed = self.data[2]
            self.moveable = False