import classes.level_controller as lc
import classes.game_driver as gd
import classes.extras as ex
import classes.simple_vector as sv
import classes.board
import random
import pygame
from math import sqrt


class Board(gd.BoardGame):
    def __init__(self, mainloop, speaker, config,  screen_w, screen_h):
        self.level = lc.Level(self,1,1)
        gd.BoardGame.__init__(self,mainloop,speaker,config,screen_w,screen_h,11,9)
        
        #self.brush_size  = 5
        self.max_size = 99
        
        
    def create_game_objects(self, level = 1):
        #create non-movable objects
        self.active_tool = 0
        self.brush_size = 5
        self.var_brush = 1
        #s = random.randrange(30, 80)
        #v = random.randrange(200, 255)
        #h = random.randrange(0, 255)
        #color = ex.hsv_to_rgb(h,s,v)
        self.bg_color = [255,255,255]
        color = [255,255,255]
        data = [0,25]
        #stretch width to fit the screen size
        data[0] = self.get_x_count(data[1],even=None)
        self.data = data
        
        self.mainloop.info.hide_buttons(0,0,0,0,1,1,1)
        
        self.layout.update_layout(data[0],data[1])
        scale = self.layout.scale
        self.board.level_start(data[0],data[1],scale)
        
        #canvas
        self.board.add_unit(3,0,data[0]-3,data[1]-3,classes.board.Letter,"",color,"",2)
        self.canvas_block = self.board.ships[0]
        self.canvas_block.set_outline([0,54,229],1)
        #tools        
        images = ["paint_pencil.png","paint_brush.png","paint_wide_brush.png","paint_rect.png","paint_circle.png","paint_eraser.png","paint_bucket.png"]
        #captions = ["pencil","brush","brush","rectangle","circle","eraser","fill"]
        for i in range(0,data[1]-4,3):
            self.board.add_unit(0,i,3,3,classes.board.ImgShip,"",color,images[i//3])

        #brush size
        self.board.add_unit(0,data[1]-4,1,1,classes.board.Letter,"-",color,"",0)
        self.board.add_unit(1,data[1]-4,1,1,classes.board.Label,"5",color,"",1)
        self.board.add_unit(2,data[1]-4,1,1,classes.board.Letter,"+",color,"",0)
        self.board.ships[-1].readable = False
        self.board.add_door(0,0,3,3,classes.board.Door,"",color,"")
        self.board.add_door(0,data[1]-3,1,1,classes.board.Door,"",color,"")
        tool_len = len(self.board.ships)
        tool_max = tool_len - 3
        #color pallette
        h = 0
        s = 250
        v = 100
        #number of available color spaces minus 2 for black and white
        number_of_colors = data[0]*3 - 2
        number_of_hues = 13
        number_of_col_per_hue = number_of_colors // number_of_hues
        if number_of_col_per_hue > 3:
            v_num = (255-v)//(number_of_col_per_hue-3)
        else:
            v_num = 150
        
        #greyscale
        grey_num = number_of_colors+2 - number_of_hues * number_of_col_per_hue
        if grey_num > 1:
            grey_v_num = (255 // (grey_num-1))
        else:
            grey_v_num = 0
        grey_count = 0
        for j in range(data[1]-3,data[1]):
            for i in range(data[0]):
                color2 = ex.hsv_to_rgb(h,s,v)
                self.board.add_unit(i,j,1,1,classes.board.Letter,"",color2,"",2)
                #self.board.ships[-1].highlight = False
                if h < 255:
                    if v <= (255-v_num):
                        v += v_num
                    else:
                        if s > 115:
                            s -= 70
                        else:
                            v = 100
                            s = 250
                            h += 20
                if h > 255:
                    if grey_count == 0:
                        s = 0
                        v = 0
                        grey_count += 1
                    else:
                        v += grey_v_num
                        
        self.active_color = self.board.ships[tool_len+1].initcolor
        self.size_display = self.board.units[0]
        self.tool_door = self.board.units[-2]
        self.color_door = self.board.units[-1]
        self.btn_down = False
        self.paint_function = [self.paint_pencil, self.paint_brush1, self.paint_brush2, self.paint_rect, self.paint_circle, self.paint_eraser, self.paint_bucket]
        
        #points        
        self.p_first = [0,0]
        self.p_last = [0,0]
        self.p_prev = [0,0]
        self.p_current = [0,0]
                
        doors = [self.tool_door,self.color_door]
        for each in doors:
            each.door_outline = True
            self.board.all_sprites_list.move_to_front(each)
        
        for each in self.board.ships:
            each.outline = False
            each.draggable = False
            each.highlight = False
        
        self.canvas = pygame.Surface([self.canvas_block.grid_w*self.board.scale, self.canvas_block.grid_h*self.board.scale-1])
        self.canvas.fill(self.canvas_block.initcolor)
        self.canvas_org = self.canvas.copy() #pygame.Surface([self.canvas_block.grid_w*self.board.scale, self.canvas_block.grid_h*self.board.scale-1])
        
        #self.outline_all(1,1)

    def handle(self,event):
        gd.BoardGame.handle(self, event) #send event handling up
        if event.type == pygame.MOUSEBUTTONDOWN:
            # Change the x/y screen coordinates to grid coordinates
            pos = event.pos
            #column=(pos[0]-self.layout.menu_w) // (self.layout.width)
            #row=pos[1] // (self.layout.height)
            active = self.board.active_ship
            if event.button == 1:
                if active == 0:
                    self.btn_down = True
                    canvas_pos = [pos[0]-self.layout.menu_w-3*self.layout.scale,pos[1]]
                    self.p_first = canvas_pos
                    self.p_prev = canvas_pos
                    self.p_current = canvas_pos
                    self.paint_function[self.active_tool](0)
                    pygame.mouse.set_cursor(*pygame.cursors.broken_x)
                    #print(canvas_pos)
                    #print("start painting")
                    #pass #start of painting done here
                elif 0 < active < 8:
                    self.active_tool = active - 1
                    self.tool_door.set_pos(self.board.active_ship_pos)
                    #print("active_tool: {0}".format(self.active_tool))
                elif active == 8:
                    self.size_down()
                elif active == 9:
                    self.size_up()
                elif active > 9:
                    self.active_color = self.board.ships[active].initcolor
                    self.color_door.set_pos(self.board.active_ship_pos)
            elif event.button == 3:
                #if right-click on brush size -> in/decrease by 5
                if active == 8:
                    self.size_down(5)
                elif active == 9:
                    self.size_up(5)
                         
        elif event.type == pygame.MOUSEMOTION and self.btn_down == True:
            active = self.board.active_ship
            pos = event.pos
            column=(pos[0]-self.layout.menu_w) // (self.layout.width)
            row=pos[1] // (self.layout.height)
            if active == 0 and column > 2 and row < self.data[1]-3:
                canvas_pos = [pos[0]-self.layout.menu_w-3*self.layout.scale,pos[1]]
                self.p_prev = self.p_current
                self.p_current = canvas_pos
                self.paint_function[self.active_tool](1)
            
        elif event.type == pygame.MOUSEBUTTONUP and event.button == 1:
            
            active = self.board.active_ship
            pos = event.pos
            column=(pos[0]-self.layout.menu_w) // (self.layout.width)
            row=pos[1] // (self.layout.height)
            if active == 0 and column > 2 and row < self.data[1]-3:
                #drop the new object onto the painting
                canvas_pos = [pos[0]-self.layout.menu_w-3*self.layout.scale,pos[1]]
                self.p_last = canvas_pos
                self.paint_function[self.active_tool](2)
            else:
                if self.btn_down:
                    self.screen_restore()
                    self.copy_to_screen()
            self.btn_down = False
            
    def size_up(self,step=1):
        if self.brush_size < self.max_size-step+1:
            self.brush_size += step       
            self.size_display.value = str(self.brush_size)
        else:
            self.brush_size = self.max_size  
            self.size_display.value = str(self.brush_size)
        
    def size_down(self,step=1):
        if self.brush_size > step:
            self.brush_size -= step
            self.size_display.value = str(self.brush_size)
        else:
            self.brush_size = 0
            self.size_display.value = str(self.brush_size)
        
    #states => mouse states => 0 - mouse_btn_down, 1 - mouse_move, 2 - mouse_btn_up
    def paint_pencil_old(self,state):
        if self.brush_size > 0:
            if state == 0:
                self.backup_canvas()
                pygame.draw.circle(self.canvas, self.active_color, self.p_current, self.brush_size//2,0)
                self.copy_to_screen()
            elif state == 1:
                pygame.draw.line(self.canvas, self.active_color, self.p_prev, self.p_current,self.brush_size)
                #smooth corners            
                if self.brush_size > 2:
                    if self.brush_size % 2 == 0:
                        r = self.brush_size // 2
                    else:
                        r = self.brush_size // 2 - 1
                    if r>3: r -= 2
                    pygame.draw.circle(self.canvas, self.active_color, self.p_current, r,0)
                self.copy_to_screen()
    
    def paint_pencil(self,state):
        if self.brush_size > 0:
            if state == 0:
                self.backup_canvas()
                
                #pygame.draw.line(self.canvas, self.active_color, [100,100], [300,300],45)
                #pygame.draw.line(self.canvas, self.active_color, [200,100], [400,100],45)
                #pygame.draw.line(self.canvas, self.active_color, [200,200], [400,200],45)
                pygame.draw.circle(self.canvas, self.active_color, self.p_current, self.brush_size//2,0)
                self.copy_to_screen()
            elif state == 1:
                width = self.brush_size
                if self.brush_size > 2:
                    if self.brush_size % 2 == 0:
                        r = self.brush_size // 2
                        width = self.brush_size +3
                    else:
                        r = self.brush_size // 2# - 1
                        width = self.brush_size+2
                        
                    pygame.draw.circle(self.canvas, self.active_color, self.p_current, r,0)
                pygame.draw.line(self.canvas, self.active_color, self.p_prev, self.p_current,width)
                self.copy_to_screen()
                
    def paint_brush1(self,state):
        if self.brush_size > 0:
            if state == 0:
                self.backup_canvas()
                
                #pygame.draw.line(self.canvas, self.active_color, [100,100], [300,300],45)
                #pygame.draw.line(self.canvas, self.active_color, [200,100], [400,100],45)
                #pygame.draw.line(self.canvas, self.active_color, [200,200], [400,200],45)
                pygame.draw.circle(self.canvas, self.active_color, self.p_current, self.var_brush//2,0)
                self.copy_to_screen()
            elif state == 1:
                if self.var_brush < self.brush_size:
                    self.var_brush +=1
                pygame.draw.line(self.canvas, self.active_color, self.p_prev, self.p_current,self.var_brush)
                pygame.draw.circle(self.canvas, self.active_color, self.p_current, self.var_brush//2,0)
                self.copy_to_screen() 
            elif state == 2:
                self.var_brush = 1
                
    def paint_brush2(self,state):
        if self.brush_size > 0:
            if state == 0:
                self.backup_canvas()
                #pygame.draw.circle(self.canvas, self.active_color, self.p_current, self.brush_size//2,0)
                #self.copy_to_screen()
            elif state == 1:
                if self.brush_size > 5:
                    step = 3
                else:
                    step = 1
                for i in range(-(self.brush_size//2),self.brush_size//2,step):                
                    pygame.draw.line(self.canvas, self.active_color, [self.p_prev[0]-i,self.p_prev[1]], [self.p_current[0]-i,self.p_current[1]],5)
                self.copy_to_screen() 
                
    def paint_brush2_broken(self,state):
        if self.brush_size > 0:
            if state == 0:
                self.backup_canvas()
                #pygame.draw.circle(self.canvas, self.active_color, self.p_current, self.brush_size//2,0)
                pygame.draw.ellipse(self.canvas, self.active_color, [int(self.p_current[0])-self.brush_size//2,int(self.p_current[1])-self.brush_size//4,self.brush_size,self.brush_size//2],0)
                self.copy_to_screen()
            elif state == 1:
                lt = [min(self.p_prev[0],self.p_current[0]),min(self.p_prev[1],self.p_current[1])]
                rb = [max(self.p_prev[0],self.p_current[0]),max(self.p_prev[1],self.p_current[1])]
                #rectangle = [lt[0],lt[1],rb[0]-lt[0],rb[1]-lt[1]]
                dist = sv.Vector2.from_points(lt, rb)
                dist_max = max(dist)
                if dist[0] > dist[1]:
                    inc = [dist[0],dist[1]/dist[0]]
                else:
                    inc = [(dist[0]/dist[1]),dist[1]]
                
                x = self.p_prev[0]
                y = self.p_prev[1]
                for i in range(int(dist_max)):
                    x = self.p_prev[0] + int(inc[0]*i)
                    y = self.p_prev[1] + int(inc[1]*i)
                #pygame.draw.ellipse(self.canvas, self.active_color, [int(self.p_current[0])-self.brush_size//2,int(self.p_current[1])-self.brush_size//4,self.brush_size,self.brush_size//2],0)         
                pygame.draw.ellipse(self.canvas, self.active_color, [x-self.brush_size//4,y-self.brush_size//2,self.brush_size,self.brush_size//2],0)         
                self.copy_to_screen() 
                
    def paint_brush2_broken2(self,state):
        if self.brush_size > 0:
            if state == 0:
                self.backup_canvas()
                #pygame.draw.circle(self.canvas, self.active_color, self.p_current, self.brush_size//2,0)
                pygame.draw.ellipse(self.canvas, self.active_color, [int(self.p_current[0])-self.brush_size//2,int(self.p_current[1])-self.brush_size//4,self.brush_size//2,self.brush_size],0)      
                self.copy_to_screen()
            elif state == 1:
                lt = [min(self.p_prev[0],self.p_current[0]),min(self.p_prev[1],self.p_current[1])]
                rb = [max(self.p_prev[0],self.p_current[0]),max(self.p_prev[1],self.p_current[1])]
                #rectangle = [lt[0],lt[1],rb[0]-lt[0],rb[1]-lt[1]]
                dist = sv.Vector2.from_points(lt, rb)
                a = dist[1]
                b = dist[0]
                c = sqrt(a*a+b*b)
                r1 = self.brush_size
                r2 = self.brush_size // 2
                if b==0: b=1
                if c==0: c=1
                #r_line = int((r1*r2)/(sqrt((r2*b/c)*(r2*b/c)+(r1*a/c)*(r1*a/c))))
                div = ((r2*a/b)*(r2*a/b)+(r1*b/c)*(r1*b/c))
                if div == 0: div = 1
                r_line = int((r1*r2)/sqrt(div))
                #if self.p_prev[0]
                pygame.draw.line(self.canvas, self.active_color, self.p_prev, self.p_current,r_line*2)
                pygame.draw.circle(self.canvas, self.active_color, self.p_current, r_line,0)
                #pygame.draw.ellipse(self.canvas, self.active_color, [int(self.p_current[0])-self.brush_size//4,int(self.p_current[1])-self.brush_size//2,self.brush_size//2,self.brush_size],0)            
                self.copy_to_screen() 
        
    def paint_rect(self,state):
        if state == 0:
            self.backup_canvas()
        elif state == 1:
            lt = [min(self.p_first[0],self.p_current[0]),min(self.p_first[1],self.p_current[1])]
            rb = [max(self.p_first[0],self.p_current[0]),max(self.p_first[1],self.p_current[1])]
            rectangle = [lt[0],lt[1],rb[0]-lt[0],rb[1]-lt[1]]
            dist = sv.Vector2.from_points(lt, rb)
            if min(dist) > 2:
                self.screen_restore()
                pygame.draw.rect(self.canvas, self.active_color, rectangle, 1)
                self.copy_to_screen()
        elif state == 2:
            lt = [min(self.p_first[0],self.p_last[0]),min(self.p_first[1],self.p_last[1])]
            rb = [max(self.p_first[0],self.p_last[0]),max(self.p_first[1],self.p_last[1])]
            rectangle = [lt[0],lt[1],rb[0]-lt[0],rb[1]-lt[1]]
            if self.brush_size > 0:
                self.screen_restore()
                dist = sv.Vector2.from_points(lt, rb)
                dist = self.v2_to_int(dist)
                if min(dist)//2 > self.brush_size:
                    border_width = self.brush_size
                    if border_width > 4:
                        borders = []
                        if border_width % 2 == 0:
                            bw2=border_width // 2 - 1
                        else:
                            bw2=border_width // 2
                        border_rects = []
                        #top
                        border_rects.append([lt[0]-bw2,lt[1]-bw2,dist[0]+border_width-1,border_width])
                        #bottom
                        border_rects.append([rb[0]-dist[0]-bw2,rb[1]-bw2,dist[0]+border_width-1,border_width])
                        for each in border_rects:
                            pygame.draw.rect(self.canvas, self.active_color, each, 0)
                    
                else:
                    border_width = 0
            else:
                border_width = 0
            #rectangle = [self.p_first[0], self.p_first[1], self.p_last[0]-self.p_first[0],self.p_last[1]-self.p_first[1]]            
            pygame.draw.rect(self.canvas, self.active_color, rectangle, border_width)
            self.copy_to_screen()

    def paint_circle(self,state):
        if state == 0:
            self.backup_canvas()
        elif state == 1:
            lt = [min(self.p_first[0],self.p_current[0]),min(self.p_first[1],self.p_current[1])]
            rb = [max(self.p_first[0],self.p_current[0]),max(self.p_first[1],self.p_current[1])]
            rectangle = [lt[0],lt[1],rb[0]-lt[0],rb[1]-lt[1]]
            dist = sv.Vector2.from_points(lt, rb)
            if min(dist) > 2:
                self.screen_restore()
                pygame.draw.rect(self.canvas, self.active_color, rectangle, 1)
                pygame.draw.ellipse(self.canvas, self.active_color, rectangle, 1)
                self.copy_to_screen()
        elif state == 2:
            lt = [min(self.p_first[0],self.p_last[0]),min(self.p_first[1],self.p_last[1])]
            rb = [max(self.p_first[0],self.p_last[0]),max(self.p_first[1],self.p_last[1])]
            rectangle = [lt[0],lt[1],rb[0]-lt[0],rb[1]-lt[1]]
            if self.brush_size > 0:
                dist = sv.Vector2.from_points(lt, rb)
                if min(dist)//2 > self.brush_size+2:
                    border_width = self.brush_size
                else:
                    border_width = 0
            else:
                border_width = 0
            #rectangle = [self.p_first[0], self.p_first[1], self.p_last[0]-self.p_first[0],self.p_last[1]-self.p_first[1]]
            self.screen_restore()
            pygame.draw.ellipse(self.canvas, self.active_color, rectangle, border_width)
            if border_width > 4:
                fillup_rect1 = [lt[0]+1,lt[1]+1,rb[0]-lt[0]-0.5,rb[1]-lt[1]-0.5]
                fillup_rect2 = [lt[0]+1,lt[1]+0.5,rb[0]-lt[0]-1,rb[1]-lt[1]-0.5]
                fillup_rect3 = [lt[0]+0.5,lt[1]+1,rb[0]-lt[0]-0.5,rb[1]-lt[1]-1]
                pygame.draw.ellipse(self.canvas, self.active_color, fillup_rect1, border_width-1)
                pygame.draw.ellipse(self.canvas, self.active_color, fillup_rect2, border_width)
                pygame.draw.ellipse(self.canvas, self.active_color, fillup_rect2, border_width)
            self.copy_to_screen()
            
    def paint_circle_old(self,state):
        if state == 0:
            self.backup_canvas()
        elif state == 1:
            pass
        elif state == 2:
            lt = [min(self.p_first[0],self.p_last[0]),min(self.p_first[1],self.p_last[1])]
            rb = [max(self.p_first[0],self.p_last[0]),max(self.p_first[1],self.p_last[1])]
            rectangle = [lt[0],lt[1],rb[0]-lt[0],rb[1]-lt[1]]
            pygame.draw.ellipse(self.canvas, self.active_color, rectangle, 0)
            self.copy_to_screen()

    def paint_eraser(self,state):
        if self.brush_size > 0:
            if state == 0:
                self.backup_canvas()
                pygame.draw.circle(self.canvas, self.bg_color, self.p_current, self.brush_size//2,0)
                self.copy_to_screen()
            elif state == 1:
                pygame.draw.line(self.canvas, self.bg_color, self.p_prev, self.p_current,self.brush_size)
                pygame.draw.circle(self.canvas, self.bg_color, self.p_current, self.brush_size//2,0)
                self.copy_to_screen()
  

    def paint_bucket(self,state):
        if state == 0:
            self.backup_canvas()
            self.canvas.fill(self.active_color)
            self.bg_color = self.active_color
            self.copy_to_screen()
            
    def backup_canvas(self):
        self.canvas_org = self.canvas_block.painting.copy()

    def copy_to_screen(self):
        self.canvas_block.painting = self.canvas.copy()
        self.mainloop.redraw_needed[0] = True

    def screen_restore(self):
        self.canvas = self.canvas_org.copy()
        
    def update(self,game):
        game.fill((255,255,255))
        gd.BoardGame.update(self, game) #rest of painting done by parent
        
    def v2_to_int(self,vector):
        integers = [int(each) for each in vector]
        return integers
        
    def check_result(self):
        pass