import classes.level_controller as lc
import classes.game_driver as gd
import classes.extras as ex
import pygame

import classes.board
import random
import colorsys
from math import pi,cos,sin

class Board(gd.BoardGame):
    def __init__(self, mainloop, speaker, config, screen_w, screen_h):
        self.level = lc.Level(self,99,3)
        gd.BoardGame.__init__(self,mainloop,speaker,config,screen_w,screen_h,9,5)
        
        
    def create_game_objects(self, level = 1):
        #create non-movable objects
        s = random.randrange(100, 150, 5)
        v = random.randrange(230, 255, 5)
        h = random.randrange(0, 255, 5)
        color = ((255,255,255))
        white = ((255,255,255))
        color0 = ex.hsv_to_rgb(h,40,230) #highlight 1
        color1 = ex.hsv_to_rgb(h,s,v) #highlight 2

        if self.level.lvl == 1:
            data = [9,5,3,5,2,3]
        elif self.level.lvl == 2:
            data = [9,5,3,7,2,3]
        elif self.level.lvl == 3:
            data = [9,5,3,10,2,3]
            
        self.data = data
        #self.layout.update_layout(data[0],data[1])
        self.board.level_start(data[0],data[1],self.layout.scale)
        
        self.num_list = []
        self.num_list2 = []
        
        sign = "/"        
        numbers = []
        for i in range(data[4]):
            num1 = random.randrange(1,data[3]-1)
            num2 = random.randrange(num1+1,data[3])
            numbers.append([num1,num2])
            expr = str(float(num1)) + sign + str(float(num2))
            disp = str(num1) + sign + str(num2)
            self.num_list.append(expr)
            self.num_list2.append(disp)
        
        #create table to store 'binary' solution 
        #self.solution_grid = [1 for x in range(data[0])]
        self.solution_grid = [1,1,1,1,1,1,1,1,1]
        self.expression = [" " for x in range(data[0])]
        #find position of first door square
        xd = (data[0]-data[2])//2

        #add objects to the board
        self.board.add_unit(0,1,3,3,classes.board.Label,"",white,"",data[5])
        self.board.add_unit(6,1,3,3,classes.board.Label,"",white,"",data[5])
        for i in range(0,data[4]):
            x2 = xd+i*2
            caption = self.num_list2[i]
            self.board.add_unit(x2,2,1,1,classes.board.Label,caption,color1,"",data[5])
            self.board.units[i+2].set_outline(0,1)
            self.expression[x2]=str(self.num_list[i])
            if i< data[4]-1:            
                self.solution_grid[x2+1]=1

        signs = [" < "," = "," > "]*(data[4]-1)
        if self.level.lvl > 12: signs.append(" < ") #just for the symetry

        for i in range(len(signs)):
            if len(signs) < data[0]:
                if i==0 and len(signs)%2 == 0:
                    x=data[0]//2
                    y=3
                else:
                    x = (data[0]-len(signs))//2
                    y=0
            else:
                if i < data[0]:
                    x=0
                    y=0
                else:
                    x = ((data[0] - (len(signs) - data[0]))//2)-data[0]
                    y=3

            self.board.add_unit(x+i,y,1,1,classes.board.Letter,signs[i],color1,"",data[5])
            self.board.ships[i].readable = False
            self.board.ships[i].set_outline(0,1)
            
        #self.outline_all(0,1)
        ind = len(self.board.units)
        for i in range(0,data[4]-1):
            self.board.add_door(xd+i*2+1,2,1,1,classes.board.Door,"",white,"")
            self.board.units[ind + i].door_outline = True   
            self.board.all_sprites_list.move_to_front(self.board.units[ind + i])      

        
        instruction = "Drag one of the <, > or = (greater, lesser or equal) to the red square."
        self.board.add_unit(0,data[1]-1,data[0],1,classes.board.Label,instruction,color0,"",9)
        self.board.units[-1].set_outline(0,1)
        if self.level.lvl == 1 and self.level.game_step == 1 and self.config.read_inst:        
            self.say("Drag one of the greater, lesser or equal  to the red square.")   
        
        size = self.board.units[0].grid_w*self.board.scale     
        center = [size//2,size//2] 
        for i in range(2):
            canvas = pygame.Surface([size, size-1])
            canvas.fill(self.board.units[i].initcolor)
            self.draw_circles(numbers[i],canvas,size,center,color1)#data[7](data, canvas, i)
            self.board.units[i].painting = canvas.copy()
        
        
        
    def draw_circles(self,numbers,canvas,size,center,color):
        angle_step = 2*pi/numbers[1]
        angle_start= -pi/2
        angle_arc_start = -pi/2
        r = size//2 - 10
        angle = angle_start
        angle_s = angle_arc_start
        angle_e = angle_arc_start + numbers[0]*2*pi/numbers[1]

        #pygame.draw.arc(canvas, ((30,230,30)), (10,10,size-20,size-20), -angle_e, -angle_s, r)
        #manually draw the arc - the 100% width of the arc does not impress
        i = 0
        while angle < angle_e: #maximum of 158 lines per pi
            x=(r-2)*cos(angle)+center[0]
            y=(r-2)*sin(angle)+center[1]
            pygame.draw.line(canvas,color,[center[0],center[1]],[x,y],5)
            i += 1
            angle = angle_start + 0.02*(i)

        pygame.draw.ellipse(canvas, ((100,100,100)), (10,10,size-20,size-20),4)
        
        for i in range(numbers[1]):
            #angle for line
            angle = angle_start + angle_step*i
            #angle for arc

            #pygame.draw.arc(canvas, ((70,70,70)), (10,10,size-20,size-20), pi, 0, 2)
            
            # Calculate the x,y for the end point
            x=r*cos(angle)+center[0]
            y=r*sin(angle)+center[1]
            
            # Draw the line from the center to the calculated end point
            pygame.draw.line(canvas,((100,100,100)),[center[0],center[1]],[x,y],2)
            
            
    def handle(self,event):
        gd.BoardGame.handle(self, event) #send event handling up

    def update(self,game):
        game.fill((255,255,255))
        gd.BoardGame.update(self, game) #rest of painting done by parent

    def check_result(self):
        if self.changed_since_check:
            if self.board.grid[2] == self.solution_grid:
                for i in range(len(self.board.ships)):
                	if self.board.ships[i].grid_y == 2: #if the sign is on line with expression
                	    value = self.board.ships[i].value
                	    if value == " = ":
                	        value = "=="
                	    self.expression[self.board.ships[i].grid_x]=value
                eval_string = s = ''.join(self.expression)
                eval_string.strip()
                if eval(eval_string)==True:
                    self.level.next_board()
                else:
                    self.say("Sorry! It is wrong.",6)
                    self.changed_since_check = False
                    