import classes.level_controller as lc
import classes.game_driver as gd
import classes.extras as ex
import pygame

import classes.board
import random
import colorsys

class Board(gd.BoardGame):
    def __init__(self, mainloop, speaker, config, screen_w, screen_h):
        self.level = lc.Level(self,5,10)
        gd.BoardGame.__init__(self,mainloop,speaker,config,screen_w,screen_h,20,10)
        
    def create_game_objects(self, level = 1):
        #create non-movable objects
        self.board.draw_grid = False
        s = random.randrange(150, 190, 5)
        v = random.randrange(230, 255, 5)
        h = random.randrange(0, 255, 5)
        color0 = ex.hsv_to_rgb(h,40,230) #highlight 1
        color1 = ex.hsv_to_rgb(h,70,v) #highlight 2
        color2 = ex.hsv_to_rgb(h,s,v) #normal color
        color3 = ex.hsv_to_rgb(h,230,100)
        white = ((255,255,255))

        #data = [x_count, y_count, number of items on the list, top_quntity,font-size]
        if self.level.lvl == 1: #images 42x42
            data = [20,14,3,3,2]
        elif self.level.lvl == 2:
            data = [20,14,3,5,2]
        elif self.level.lvl == 3:
            data = [20,14,3,7,2]
        elif self.level.lvl == 4:
            data = [20,14,4,3,2]
        elif self.level.lvl == 5:
            data = [20,14,4,5,2]
        elif self.level.lvl == 6:
            data = [20,14,4,7,2]
        elif self.level.lvl == 7:
            data = [20,14,5,3,2]
        elif self.level.lvl == 8:
            data = [20,14,5,5,2]
        elif self.level.lvl == 9:
            data = [20,14,6,3,2]
        elif self.level.lvl == 10:
            data = [20,14,6,5,2]
            
        #rescale the number of squares horizontally to better match the screen width
        data[0] = self.get_x_count(data[1],even=None)
        self.data = data
        self.layout.update_layout(data[0],data[1])
        self.board.level_start(data[0],data[1],self.layout.scale)
        
        #shopping list
        #self.board.add_door(data[0]-6,0,6,7,classes.board.Door,"",white,"")
        #self.board.units[0].door_outline = True  
        
        #basket
        self.board.add_door(data[0]-6,data[1]-6,6,5,classes.board.Door,"",white,"")
        self.board.units[0].door_outline = True
        #basket image - 260 x 220
        self.board.add_door(data[0]-6,data[1]-6,6,5,classes.board.Door,"",white,"basket.png")

        
        self.board.add_unit(data[0]-6,0,6,1,classes.board.Label,"Shopping List",white,"",data[4]+1)
        #images = ["apple_42.png","apple2_42.png","strawberry_42.png","pear_42.png","orange_42.png","onion_42.png","tomato_42.png","watermelon_42.png","lemon_42.png","cherry_42.png","pepper_42.png","carrot_42.png","banana_42.png"]
        f_end = "_42.png"
        items = ["green_apple","red_apple","strawberry","pear","orange","onion","tomato","lemon","cherry","pepper","carrot","banana","watermelon"]
        self.items = items        
        multiple_items = ["green apples","red apples","strawberries","pears","oranges","onions","tomatoes","lemons","cherries","peppers","carrots","bananas","watermelons"]     
        
        item_indexes = [x for x in range(len(items))]
        #quantities = [x for x in range(1,data[3]+1)]
        self.chosen_items = [[],[]]
        self.solution = {}
        #pick items and quantities
        for i in range(data[2]):
            index = random.randrange(0,len(item_indexes))
            self.chosen_items[0].append(item_indexes[index])
            quantity = random.randrange(1,data[3]+1)
            self.chosen_items[1].append(quantity)
            self.solution[str(item_indexes[index])] = quantity
            del(item_indexes[index])
            
        #create shopping list
        for i in range(data[2]):
            ind = self.chosen_items[0][i]
            if self.chosen_items[1][i]>1:
                caption = multiple_items[ind]
            else:
                caption = items[ind]
            self.board.add_unit(data[0]-6,i+1,1,1,classes.board.Label,str(self.chosen_items[1][i]) +" "+ chr(215),white,"",data[4])
            self.board.add_unit(data[0]-5,i+1,1,1,classes.board.ImgShip,"",white,items[ind]+f_end,data[4])
            self.board.add_unit(data[0]-4,i+1,4,1,classes.board.Label,caption,white,"",data[4])
            self.board.ships[i].draggable = False
            self.board.ships[i].outline = False
        #rearange z-order of red outlines (shopping list and basket)
        for i in range(2):
            self.board.all_sprites_list.move_to_front(self.board.units[i]) 
        #self.board.all_sprites_list.move_to_back(self.board.units[3]) 
        #adjust number of items on shelf so the items are closer to the target square
        shelf_len = 10 + data[0]-20
        #put stuff on shelves:
        for i in range(len(items)):
            image = items[i]+f_end
            #put 10 pieces each
            for j in range(shelf_len):
                self.board.add_unit(j,i,1,1,classes.board.ImgShip,items[i],white,image,data[4])
        
        self.board.all_sprites_list.move_to_front(self.board.units[0]) 
        instruction = "Check the shopping list and drag all needed items into the basket."
        self.board.add_unit(0,data[1]-1,data[0],1,classes.board.Label,instruction,color0,"",3)
        self.board.units[-1].set_outline(0, 1)
        
        #self.board.all_sprites_list.move_to_back(self.board.board_bg)


    def handle(self,event):
        gd.BoardGame.handle(self, event) #send event handling up
            
    def update(self,game):
        game.fill((255,255,255))
        gd.BoardGame.update(self, game) #rest of painting done by parent

    def check_result(self):
        if self.changed_since_check:
            #checking what sprites collide with the basket sprite
            purchased = pygame.sprite.spritecollide(self.board.units[1], self.board.ship_list, False, collided = None)
            result = {}
            #count each item and check if they are the items from the shopping list
            for i in range(len(self.items)):
                count = 0
                for each in purchased:
                    if each.value == self.items[i]:
                        count += 1
                if count > 0:
                    result[str(i)] = count
            if result == self.solution:
                self.level.next_board()
            else:
                self.say("Sorry! You messed something up!",6)
                self.changed_since_check = False
