import classes.level_controller as lc
import classes.game_driver as gd
import classes.extras as ex

import pygame
import classes.board
import random
import colorsys


class Board(gd.BoardGame):
    def __init__(self, mainloop, speaker, config,  screen_w, screen_h):
        self.level = lc.Level(self,1,1)
        gd.BoardGame.__init__(self,mainloop,speaker,config,screen_w,screen_h,11,9)
        
        
    def create_game_objects(self, level = 1):
        #create non-movable objects
        self.board.draw_grid = False
        s = random.randrange(30, 80)
        v = random.randrange(200, 255)
        h = random.randrange(0, 225)
        #ex.hsv_to_rgb(h,s,v)
        #rgb = colorsys.hsv_to_rgb(h,s,v)
        letter_color = ex.hsv_to_rgb(h,s,v) #[round(each*255) for each in rgb]
        outline_color =  ex.hsv_to_rgb(h,s+50,v-50)        
        frame_color = [255,255,255]
        card_color = ex.hsv_to_rgb(h+10,s-25,v)

        data = [14,10]
        #stretch width to fit the screen size
        data[0] = self.get_x_count(data[1],even=True)
        if data[0]<14:
            data[0]=14
        self.data = data
        self.mainloop.info.hide_buttons(0,0,0,0,1,0,1)
        self.layout.update_layout(data[0],data[1])
        scale = self.layout.scale
        self.board.level_start(data[0],data[1],scale)
        
        self.word_list = ['one', 'two', 'three', 'four', 'five', 'six', 'seven', 'eight', 'nine', 'ten', 'eleven', 'twelve', 'thirteen', 'fourteen', 'fifteen', 'sixteen', 'seventeen', 'eighteen', 'nineteen', 'twenty']

        x = 0
        x2 = (data[0] - (20 - data[0]))//2
        y = 0
        
        for i in range(1,21):
            self.board.add_unit(x,y,1,1,classes.board.Letter,str(i),letter_color,"",2)
            self.board.ships[i-1].draggable = False
            self.board.ships[i-1].set_outline(outline_color,1)
            #self.board.ships[i-1].readable = False
            x += 1
            if x >= data[0]:
                x = x2
                y = data[1]-1

        x=(data[0]-4)//2
        y=1
        


        self.board.add_unit(x-2,y+1,2,2,classes.board.Label,"1",card_color,"",0)
        self.board.add_unit(x-2,y+3,2,2,classes.board.Label,"1",card_color,"",12)
        self.board.add_unit(x-2,y+5,2,2,classes.board.Label,"1",card_color,"",13)
        #frame size 432 x 288
        self.board.add_unit(x,y+1,6,4,classes.board.MultiImgSprite,self.word_list[0],card_color,"flashcard_numbers.png")
        self.board.add_unit(x,y+5,6,1,classes.board.Letter,self.word_list[0],card_color,"",2)
        self.board.add_unit(x,y+6,6,1,classes.board.Letter,self.word_list[0],card_color,"",11)

        #Frame
        self.board.add_unit(0,y,x-2,data[1]-2,classes.board.Obstacle,"",frame_color)
        self.board.add_unit(x+6,y,x-2,data[1]-2,classes.board.Obstacle,"",frame_color)
        self.board.add_unit(x-2,y,8,1,classes.board.Obstacle,"",frame_color)
        self.board.add_unit(x-2,y+7,8,1,classes.board.Obstacle,"",frame_color)
        
        self.board.add_door(x-2,y+1,8,6,classes.board.Door,"",card_color,"")
        #self.board.add_door(x,y+2,4,3,classes.board.Door,"",card_color,"")
        self.board.units[7].door_outline = True
        #self.board.units[11].door_outline = True
        self.board.all_sprites_list.move_to_front(self.board.units[7])
        #self.board.all_sprites_list.move_to_front(self.board.units[11])
        self.slide = self.board.ships[20]
        self.slide.build_frame_flow(20) 
        self.slide.correction = True 
        self.slide.perm_outline = True

    def handle(self,event):
        gd.BoardGame.handle(self, event) #send event handling up
        if event.type == pygame.MOUSEBUTTONUP:
            self.active_item = self.board.ships[self.board.active_ship]
            if self.active_item.unit_id < 20:
                self.create_card(self.active_item)

    def create_card(self, active):
        #self.say(active.value[0])
        self.board.units[0].value = active.value
        self.board.units[1].value = active.value
        self.board.units[2].value = active.value
        self.board.ships[20].value = self.word_list[active.unit_id]
        self.board.ships[21].value = self.word_list[active.unit_id]
        self.board.ships[22].value = self.word_list[active.unit_id]
        self.mainloop.redraw_needed[0] = True
        self.slide.set_frame(active.unit_id)
        self.board.active_ship = -1
        
    def update(self,game):
        game.fill((255,255,255))
        gd.BoardGame.update(self, game) #rest of painting done by parent

    def check_result(self):
        pass