# -*- coding: utf8 -*-
"""
Laby , par Mehdi Cherti 2010(mehdidc): 
    - generation d'un labyrinthe
    - utilisation de l'algorithme astar pour trouver le a_path le plus court(selection de la destination avec la souris)

Laby, 2010 by Mehdi Cherti(mehdidc):
- Generation of a labyrinth
- Use of astar algorithm to find the shortest path(selection of the destination with the mouse)
downloaded from:
    http://www.pythonfrance.com/codes/GENERATION-LABYRINTHE-AVEC-RECHERCHE-CHEMIN-PLUS-COURT-AVEC_51293.aspx
Rebuild and translated by Ireneusz Imiolek
"""
from  random import randint , choice
import pygame
from pygame.locals import *
import game_boards.def_const as def_const
import classes.board
import os
from math import sqrt

const = def_const.def_const()
""" definitions des constantes """

# Couleurs
const.white  =(255 , 255 , 255)
const.pink   =(255 , 0 , 255)
const.black  =(0 , 0 , 0)
const.yellow =(255 , 255 , 0)

#directions 
const.right = 0
const.left = 1
const.up = 2
const.down = 3

# autres

#const.wc = 30 # width of a single tile of laby
#const.hc = 30# height of a single tile of laby

#const.person_radius = const.wc/2 # radius of the character(which is a circle)
const.time_person = 50 # Event framerate
const.time_person_poll = 20 # update framerate




class Point:
    def __init__(self,xy):
        self.x = xy[0]
        self.y = xy[1]

""" Definition de la classe Laby_cell de labyrinthe"""
"""Laby Cell Class definition"""
class Laby_cell:
    def __init__(self):
        self.state = False
        self.laby_doors = [True , True , True , True] #Right, Left, Up, Down
        
""" distance : returns the Euclidean distance between two points """
def distance(p1 , p2):
    return sqrt((p1[0]-p2[0])**2 +(p1[1]-p2[1])**2)


""" Person Class """
class Person(pygame.sprite.Sprite):
    def __init__(self,lab,x,y,scale,img_src):
        pygame.sprite.Sprite.__init__(self)
        self.x = x
        self.y = y
        self.image = pygame.Surface([scale-1, scale-1])
        self.image.fill(const.white)
        self.rect = self.image.get_rect()
        self.rect.topleft = [self.x*scale+1,self.y*scale+1]
        self.a_path = None
        self.laby = lab
        self.scale = scale
        self.radi = (self.scale//2) - self.scale//10
        self.offset = self.scale//10
        self.color = const.yellow
        self.unit_id = 0
        
        self.img_src = img_src
        if len(self.img_src) > 0:
            self.img = self.image
            self.img_pos = (0,0)
            try:    
                self.img_org = pygame.image.load(os.path.join('images', self.img_src)).convert()
                self.img = self.img_org
                self.img_rect = self.img.get_rect()
                #if image size is larger than the sprite or is too small than resize
                size_diff = self.rect.w - self.img_rect.w                
                if  size_diff < 0:
                    self.img = self.img_org = pygame.transform.smoothscale(self.img, (self.rect.w-2, self.rect.h-2))
                elif size_diff > 7:
                    self.img = self.img_org = pygame.transform.smoothscale(self.img, (self.rect.w-7, self.rect.h-7))
                
                self.img_rect = self.img.get_rect()                
                pos_x = ((board.scale-self.img_rect.w)//2)
                pos_y = ((board.scale-self.img_rect.h)//2)
                self.img_pos = (pos_x,pos_y)
            except:
                pass
                
        self.show2()
        self.image.set_colorkey(const.white)
        
        
    """ display """    
    def show2(self):
        self.image.fill(const.white)
        self.image.blit(self.img_org, self.img_pos)
        #a,b = self.radi, self.radi #const.person_radius , const.person_radius
        #pygame.draw.circle(self.image, self.color ,(self.radi + self.offset, self.radi + self.offset), self.radi)
        
        
    def show(self,buffer):
        pass        
        #self.image.fill(self.color)
        #a,b = self.radi, self.radi #const.person_radius , const.person_radius
        #pygame.draw.circle(buffer, self.color ,(round(a+self.laby.sx+self.x*self.laby.wc)+self.offset, round(b+self.laby.sy+self.y*self.laby.hc)+self.offset), 
        #self.radi)
        
    """ movement in the direction """
    def move(self,dir):
            
        if not self.laby.get_cell(self.x , self.y).laby_doors[dir]:
            if dir == const.right and self.x+1 < self.laby.w :
                self.x += 1
            if dir == const.left and self.x-1 >=0:
                self.x -= 1
            if dir == const.up and self.y-1 >=0:
                self.y -= 1
            if dir == const.down and self.y+1 < self.laby.h:
                self.y += 1

            
            
                
    """ function used by astar to treat any adjoining square """
    def traitement(self ,  list_closed , list_open , x ,  y , dir ,parent ,dest):
        if parent.laby_doors[dir] : return
        c = self.laby.get_cell(x,y)
        if c in list_closed : return
        
        if c in list_open:
            G = distance((x,y) ,(parent.x , parent.y))
            c.dir = self.laby.notdir(dir)
            if G < c.G:
                c.G = G
                c.F = c.H + c.G
                c.parent = parent
        else:
            c.parent = parent
            c.dir = self.laby.notdir(dir)
            c.G = distance((x,y) ,(parent.x , parent.y))
            c.H = distance((x,y) ,(dest.x ,dest.y))
            c.F = c.G + c.H
            list_open.append(c)
    """ returns, after finding the path with astar, a table containing the path """
    def  get_astar(self, csource , cdest):
        
        source = self.laby.get_cell(csource[0] , csource[1])
        #a_print = [cdest[0] , cdest[1]]
        #print(a_print)
        dest = self.laby.get_cell(cdest[0] , cdest[1])
        #print(dest)
        cur=dest
        a_path = []
        while cur and(cur.x != source.x or cur.y != source.y):
            if cur.x == cur.parent.x-1 :
                a_path.append(const.right)
            if cur.x == cur.parent.x+1:
                a_path.append(const.left)
            if cur.y == cur.parent.y+1:
                a_path.append(const.up)
            if cur.y == cur.parent.y-1:
                a_path.append(const.down)
            cur = cur.parent
        
        ret = []
        id = len(a_path) - 1
        while id>=0:
            ret.append(self.laby.notdir(a_path[id]))
            id-=1
        return ret
        
    """ update the position of the person """
    def poll(self):
        if self.a_path:
            self.move(self.a_path.pop(0))
        self.rect.topleft = [self.x*self.scale+1,self.y*self.scale+1]

    """ can move according to the a_path """
    def go_to(self,a_path):
        self.a_path = a_path
        
            
    """
    pathfinding algorithm Astar, destination represents the tuple of the destination
    """
    def astar(self , desti):
        dest = Point(desti)
        list_open = []
        list_closed = []


        debut = self.laby.get_cell(self.x , self.y)
        list_open.append(debut)
        debut.F = distance((self.x,self.y) ,(dest.x,dest.y))
        debut.G = 0
        debut.H = debut.F
        debut.parent = None
        
        
        while 1:
            if len(list_open) <= 0 : 
                break
            min_ , min_id= list_open[0].F,0
                    
            for id, v in enumerate(list_open[1:]):
                if v.F < min_ :
                    min_ = v.F
                    min_id = id +1
            list_closed.append(list_open[min_id])
            
            if list_open[min_id].x == dest.x and list_open[min_id].y == dest.y : break
            
            self.traitement(list_closed,list_open,list_open[min_id].x +  1 , list_open[min_id].y , const.right , list_open[min_id] , dest)
            self.traitement(list_closed,list_open,list_open[min_id].x -  1 , list_open[min_id].y , const.left , list_open[min_id] , dest)
            self.traitement(list_closed,list_open,list_open[min_id].x  , list_open[min_id].y-1 , const.up , list_open[min_id] , dest)
            self.traitement(list_closed,list_open,list_open[min_id].x  , list_open[min_id].y+1 , const.down , list_open[min_id] , dest)
            list_open.remove(list_open[min_id])

""" Laby Class """
class laby:
    def __init__(self,w,h,sx=0,sy=0,scale=30):
        self.w = w
        self.h = h
        self.Laby_cells = []
        self.wc = scale #const.wc
        self.hc = scale #const.hc
        self.sx = sx
        self.sy = sy
        self.displayed_once = True
        """ Laby_cells initialization for each Laby_cell, it initializes its position in the labyrinth """
        for v in range(self.w*self.h):
            a = Laby_cell()
            a.x = v % self.w
            a.y = v // self.w
            self.Laby_cells.append(a)
        #print(len(self.Laby_cells))
        
    """ returns the Laby_cell corresponding to the position(x, y) """
    def get_cell(self,x,y):
        return self.Laby_cells[x +  y * self.w]
        #return self.Laby_cells[(y*self.w) + x]
        
    """ return direction opposite to a direction """
    def notdir(self,dir):
        if dir == const.right : return const.left
        if dir == const.left : return const.right
        if dir == const.up : return const.down
        if dir == const.down : return const.up
        
    """ generation of the labyrinth """
    def generate_laby(self,x=-1,y=-1):
        if x==-1:
            x = randint(0,self.w-1)
            y = randint(0,self.h-1)
        cell_act = self.get_cell(x,y)
        
        if not cell_act.state :
            cell_act.state = True
            tab = []
            if x+1<self.w and not self.get_cell(x+1,y).state : tab.append((x+1,y,const.right))
            if x-1>=0  and not self.get_cell(x-1,y).state    : tab.append((x-1,y,const.left))
            if y-1>=0  and not self.get_cell(x,y-1).state   : tab.append((x  ,y-1,const.up))
            if y+1<self.h and not self.get_cell(x,y+1).state : tab.append((x  ,y+1,const.down))
            if tab:
                
                while tab:
                    C = choice(tab)
                    if not self.get_cell(C[0] , C[1]).state:
                        
                        cell = self.get_cell(C[0] , C[1])
                        cell_act.laby_doors[C[2]] = False
                        cell.laby_doors[self.notdir(C[2])] = False
                        self.generate_laby(C[0] , C[1])
                    tab.remove(C)
                return True
            else : 
                return False
        
    """ display the labyrinth """
    def show(self,buffer):
        W,H = self.wc , self.hc
        sx,sy = self.sx , self.sy
        for y in range(self.h-1):
            for x in range(self.w-1):
                c = self.get_cell(x,y)
                if c.laby_doors[const.right] :
                    pygame.draw.line(buffer , const.black ,(sx+(x+1)*W,sy+y*H),(sx+(x+1)*W,sy+(y+1)*H),3)
                if c.laby_doors[const.down] :
                    pygame.draw.line(buffer , const.black ,(sx+(x)*W,sy+(y+1)*H),(sx+(x+1)*W,sy+(y+1)*H),3)

        x = self.w - 1

        for y in range(self.h-1):
            c = self.get_cell(x,y)
            if c.laby_doors[const.down]:
                pygame.draw.line(buffer , const.black ,(sx+x*W,sy+(y+1)*H),(sx+(x+1)*W,sy+(y+1)*H),3)

        y = self.h - 1
        for x in range(self.w-1):
            c = self.get_cell(x,y)
            if c.laby_doors[const.right]:
                pygame.draw.line(buffer , const.black ,(sx+(x+1)*W,sy+(y)*H),(sx+(x+1)*W,sy+(y+1)*H),3)
        #pygame.draw.rect(buffer , const.pink ,(sx , sy , W * self.w , H * self.h),2)
        #if self.displayed_once:        
            #self.labi_to_array()
            #self.displayed_once = False
        
    """ create the labyrinth grid table twice as big as the original laby"""
    def labi_to_array(self):
        W,H = self.wc , self.hc
        sx,sy = self.sx , self.sy
        labi_grid = [[0 for x in range(0,self.w*2-1)] for y in range(0,self.h*2-1)]
        for y in range(self.h-1):
            for x in range(self.w-1):
                c = self.get_cell(x,y)
                
                if c.laby_doors[const.right] :
                    #pygame.draw.line(buffer , const.black ,(sx+(x+1)*W,sy+y*H),(sx+(x+1)*W,sy+(y+1)*H),10)
                    gx = x*2+1
                    gy = y*2
                    labi_grid[gy][gx]=1
                    if y>0:
                        labi_grid[gy-1][gx]=1
                    if y<self.h-1:
                        labi_grid[gy+1][gx]=1

                if c.laby_doors[const.down] :
                    #pygame.draw.line(buffer , const.black ,(sx+(x)*W,sy+(y+1)*H),(sx+(x+1)*W,sy+(y+1)*H),10)
                    gx = x*2
                    gy = y*2+1
                    labi_grid[gy][gx]=1
                    if x>0:
                        labi_grid[gy][gx-1]=1
                    if x<self.w-1:
                        labi_grid[gy][gx+1]=1
                        
        x = self.w - 1

        for y in range(self.h-1):
            c = self.get_cell(x,y)
            if c.laby_doors[const.down]:
                gx = x*2
                gy = y*2+1
                labi_grid[gy][gx]=1#pygame.draw.line(buffer , const.black ,(sx+x*W,sy+(y+1)*H),(sx+(x+1)*W,sy+(y+1)*H),10)
                labi_grid[gy][gx-1]=1

        y = self.h - 1
        for x in range(self.w-1):
            c = self.get_cell(x,y)
            if c.laby_doors[const.right]:
                gx = x*2+1
                gy = y*2
                labi_grid[gy][gx]=1#pygame.draw.line(buffer , const.black ,(sx+(x+1)*W,sy+(y)*H),(sx+(x+1)*W,sy+(y+1)*H),10)
                labi_grid[gy-1][gx]=1
        #pygame.draw.rect(buffer , const.pink ,(sx , sy , W * self.w , H * self.h),2)
        return labi_grid        
