#!/usr/bin/env python

#pySioGame - Educational Activity Pack 
#Copyright (C) 2012  Ireneusz Imiolek

#This program is free software: you can redistribute it and/or modify
#it under the terms of the GNU General Public License as published by
#the Free Software Foundation, either version 3 of the License, or
#(at your option) any later version.

#This program is distributed in the hope that it will be useful,
#but WITHOUT ANY WARRANTY; without even the implied warranty of
#MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
#GNU General Public License for more details.

#You should have received a copy of the GNU General Public License
#along with this program.  If not, see <http://www.gnu.org/licenses/>.

#THIS SOFTWARE IS PROVIDED "AS IS" AND ANY EXPRESSED OR IMPLIED WARRANTIES, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED WARRANTIES OF MERCHANTABILITY AND FITNESS FOR A PARTICULAR PURPOSE ARE DISCLAIMED. IN NO EVENT SHALL THE REGENTS OR CONTRIBUTORS BE LIABLE FOR ANY DIRECT, INDIRECT, INCIDENTAL, SPECIAL, EXEMPLARY, OR CONSEQUENTIAL DAMAGES (INCLUDING, BUT NOT LIMITED TO, PROCUREMENT OF SUBSTITUTE GOODS OR SERVICES; LOSS OF USE, DATA, OR PROFITS; OR BUSINESS INTERRUPTION)
#HOWEVER CAUSED AND ON ANY THEORY OF LIABILITY, WHETHER IN CONTRACT, STRICT LIABILITY, OR TORT (INCLUDING NEGLIGENCE OR OTHERWISE) ARISING IN ANY WAY OUT OF THE USE OF THIS SOFTWARE, EVEN IF ADVISED OF THE POSSIBILITY OF SUCH DAMAGE.


#standard lib modules
import pygame
import threading
import gc

import os
import inspect

#setting the working directory by locating the absolute path to the parent file of a dummy function '_path' in next line
def _path(): pass
a = os.path.split(os.path.abspath(inspect.getsourcefile(_path)))[0]
os.chdir(a)

#local game modules
import classes.game_driver
import classes.level_controller
import classes.board
import classes.menu
import classes.info_bar
import classes.speaker
import classes.colors


class Config():
    'holds some basic configuration data - screen size among others'
    def __init__(self):
        #Screen Settings
        #set this to False if you want to manually set the screen size
        self.screen_size_autodetect = True
        #self.screen_size_autodetect = False
        
        #set the display mode: True - fullscreen or False - windowed
        self.fullscreen = False
        
        #enter the size of your screen manually if not using auto-detection
        #this will be overridden if self.screen_size_autodetect is set to True
        self.fs_width = 1440 #1024
        self.fs_height = 900 #768
        
        #set total size of OS panels and window decorations on both sides - used in windowed version.
        #this will not be auto-detected
        self.os_panels_w = 2  #sum of widths of non-hiding vertical Panels (if any) and window border (1px on each side). 
        self.os_panels_h = 61 #sum of heights of non-hiding horizontal panels (ie. menu bar(s) + application bar + window bar + border, etc.).

        
        #Speaker Settings
        #user_name - if game used by only one child - if this is set the child's name will be used in sentences said by the game
        self.user_name = ""
        
        #message said at the start of the game
        self.welcome_msg = "Hello " + self.user_name + "! Welcome back in the game."
        
        #read_inst - if set to True - it will read the game instruction at the start of each first game at the first level.
        #needs some work to get it right
        self.read_inst = False
        
        #Window title
        self.window_caption = "pySioGame"

class LogoImg(pygame.sprite.Sprite):
    'holds the logo in top left corner'
    def __init__(self):
        # Call the parent's constructor
        pygame.sprite.Sprite.__init__(self)
        self.image = pygame.Surface([123,123])
        self.image.fill((70,70,70))

        self.rect = self.image.get_rect()
        self.rect.topleft = [0,0]
        self.img_src = "logo.png"
        self.img = self.image
        self.img_pos = (0,0)
        try:
            self.img = pygame.image.load(os.path.join('images', self.img_src)).convert_alpha()
        except:
            pass
        self.update()

    def update(self):
        self.image.blit(self.img, self.img_pos)


class GamePlay(threading.Thread):
    'The top most class - subclasses the Thread to keep the game and speaker in two different processes'
    'holds main loop'
    def __init__(self,speaker):
        #Call parent's initialization
        threading.Thread.__init__(self)
        
        #Create / set additional top level game objects and mainloop variables
        self.speaker = speaker
        self.config = Config()
        self.cl = classes.colors.Color()

        #self.layout_created = False
        #Create variables used to determine when screen redrawing is needed
        #redraw_needed = [0 - game, 1 - info, 2 - menu]
        self.redraw_needed = [True,True,True]
        self.game_redraw_tick = [0,0,0]
        self.flip_needed = True
        
        #'global' score holder used in one game
        self.score = 0
        
        #As long as this is False the game will run. If some action sets it to True the loop ends and so does the game.
        self.done = False
        
    def create_subsurfaces(self,game_board):
        #create subsurfaces & set some of the initial layout constraints
        self.menu = self.screen.subsurface(self.game_board.layout.menu_pos)         #menu panel
        self.menu_l = self.menu.subsurface(self.game_board.layout.menu_l_pos)       #category menu
        self.menu_r = self.menu.subsurface(self.game_board.layout.menu_r_pos)       #game selection menu - games in a category
        self.game = self.screen.subsurface(self.game_board.layout.game_pos)         #game panel - all action happens here
        self.info_bar_offset = self.screen.subsurface(self.game_board.layout.info_bar_offset_pos) #offset - empty area between game and bottom panel - if game is not high enough to fill all available area
        self.info_bar = self.info_bar_offset.subsurface(self.game_board.layout.info_bar_pos) #info panel - level control, game info, etc.
        self.misio = self.screen.subsurface(self.game_board.layout.misio_pos)       #holds and image/logo in bottom left corner - over menu

    def fs_rescale(self, info):
        'rescale the game after fullscreen toggle, this will restart the board'
        'could not get all the game objects to scale nicely'
        #pass new screen resolution
        self.game_board.layout.update_layout_fs(self.size[0], self.size[1],self.game_board.layout.x_count,self.game_board.layout.y_count)
        #load new game - create game objects        
        self.game_board.level.load_level()    
        #adjust the layout to accomodate changes to number of squares automatically added due to screen ratio change
        self.game_board.layout.update_layout(self.game_board.data[0],self.game_board.data[1])            
        self.create_subsurfaces(self.game_board)
        #info.new_game(self.game_board,self.info_bar)  
        #self.game_board.board.update_layout(self.game_board.layout.scale)
        #self.game_board.board_layout_update()
        info.new_game(self.game_board,self.info_bar)
        
    def fullscreen_toggle(self, info):
        'toggle between fullscreen and windowed version with CTRL + F'
        'game will be reloaded'
        self.redraw_needed = [True,True,True]
        if self.config.fullscreen == True:
            self.config.fullscreen = False
            self.size = self.wn_size
            self.screen = pygame.display.set_mode(self.size)
            self.fs_rescale(info)
        else:
            self.config.fullscreen = True
            self.size = self.fs_size
            self.screen = pygame.display.set_mode(self.size, pygame.FULLSCREEN)
            self.fs_rescale(info)
    
    def run(self):
        #start of this Thread
        self.speaker.join() #join speaker Thread
        self.speaker.say(self.config.welcome_msg,6) #say welcome message
        
        #initialize pygame
        pygame.init()
        
        if self.config.screen_size_autodetect:
            #override configured screen size if auto detection turned on
            display_info = pygame.display.Info()
            self.config.fs_width  = display_info.current_w
            self.config.fs_height = display_info.current_h
            
        #set screen parameters based on configuration in Config object
        self.fs_size = [self.config.fs_width, self.config.fs_height]
        self.wn_size = [self.config.fs_width - self.config.os_panels_w, self.config.fs_height - self.config.os_panels_h]

        if self.config.fullscreen == True:
            self.size = self.fs_size
            flag = pygame.FULLSCREEN
        else:
            self.size = self.wn_size
            flag = 0
            
        self.screen=pygame.display.set_mode(self.fs_size, flag)
        
        #Set title of the window
        pygame.display.set_caption(self.config.window_caption)
        
        #create a list of one sprite holding game image or logo
        self.sprites_list = pygame.sprite.RenderPlain()
        
        #create the logo object and add it to the list to render on update
        self.front_img = LogoImg()
        self.sprites_list.add(self.front_img)
        
        #create a dummy self.game_board variable to be deleted and recreated at the beginning of the main loop
        self.game_board = None

        #create game menu / game manager
        m = classes.menu.Menu(self)
        self.m = m

        #create info panel integrated with level control - holds current level/game and some buttons to change levels, etc.
        info = classes.info_bar.InfoBar(self)
        self.info = info

        #Used to manage how fast the screen updates
        clock=pygame.time.Clock()
        
        # -------- Main Program Loop ----------- #
        while self.done==False:
            # start, switch or continue a game
            #not really an implementation of a State Machine but does the job
            if m.active_game_id != m.game_started_id: #if game id changed since last frame
                if self.game_board != None: 
                    #if this is not the first start of a game - the self.game_board has been already 'created' at least once
                    self.game_board.board.clean() #empty sprite groups, delete lists
                    del(self.game_board) #delete all previous game objects
                #recreate a new game and subsurfaces
                self.game_board = m.game_constructor(self,self.speaker,self.config,self.size[0],self.size[1])
                m.game_started_id = m.active_game_id
                m.l = self.game_board.layout
                self.create_subsurfaces(self.game_board)
                info.new_game(self.game_board,self.info_bar)
                gc.collect() #force garbage collection to remove remaining variables to free memory
            
            elif self.game_board.level.lvl != self.game_board.level.prev_lvl:
                #if game id is the same but the level changed load new level
                self.create_subsurfaces(self.game_board)
                info.new_game(self.game_board,self.info_bar)
                self.game_board.level.prev_lvl = self.game_board.level.lvl
                gc.collect()

            #Process or delegate events
            mods = pygame.key.get_mods()
            for event in pygame.event.get(): #pygame.event.get(): # User did something
                if event.type == pygame.QUIT or (event.type == pygame.KEYDOWN and event.key == pygame.K_ESCAPE):
                    self.done = True #mark to finish the loop and the game
                elif event.type == pygame.KEYDOWN and (mods and pygame.KMOD_LCTRL and event.key == pygame.K_f):
                    #CTRL + F - pressed
                    self.fullscreen_toggle(info)
                elif event.type == pygame.MOUSEBUTTONDOWN or event.type == pygame.MOUSEMOTION:
                    pos = event.pos
                    if event.type == pygame.MOUSEBUTTONDOWN:
                        #pygame.mouse.set_cursor(*pygame.cursors.broken_x)
                        if self.game_board.show_msg == True:
                            #if dialog after completing the game is shown then hide it and load next game
                            self.game_board.show_msg = False
                            self.game_board.level.next_board_load()
                    if pos[0] > self.game_board.layout.menu_w and pos[1] < self.game_board.layout.game_h:
                        #clicked on game board
                        self.game_board.handle(event)
                    elif pos[0] < self.game_board.layout.menu_w:
                        #clicked on menu panel
                        if pos[0] < self.game_board.layout.menu_l_w:
                            #clicked on category menu
                            m.handle_menu_l(event)
                        else:
                            #clicked on game selection menu
                            m.handle_menu_r(event,self.game_board.layout.menu_l_w)
                    else:
                        #clicked on info panel
                        info.handle(event,self.game_board.layout,self)
                elif event.type == pygame.MOUSEBUTTONUP:
                    self.game_board.handle(event)
                    pygame.mouse.set_cursor(*pygame.cursors.arrow)
                else:
                    #let the game handle other events
                    self.game_board.handle(event)
            
            #trying to save the CPU - only update a subsurface entirely, (can't be bothered to play with dirty sprites)
            #if anything has changed on the subsurface or it's size has changed
            
            #creating list of drawing functions and arguments for each subsurface
            draw_func = [self.game_board.update,info.draw,m.draw_menu]
            draw_func_args = [[self.game],[self.info_bar, self.info_bar_offset],[self.menu,self.menu_l,self.menu_r,self.game_board.layout]]
            
            #checking if any of the subsurfaces need updating and updating them if needed
            #in reverse order so the menu is being drawn first
            for i in range(2,-1,-1):
                if self.redraw_needed[i] and self.game_redraw_tick[i] < 3:
                    draw_func[i](*draw_func_args[i])
                    self.game_redraw_tick[i] += 1
                    if self.game_redraw_tick[i] == 2:
                        self.redraw_needed[i] = False
                        self.game_redraw_tick[i] = 0
                    self.flip_needed = True
                    if i == 2:
                        #draw the logo over menu - left top corner
                        self.front_img.update()
                        self.sprites_list.draw(self.misio)            

            if self.flip_needed:
                #update the screen with what we've drawn.
                pygame.display.flip()
                self.flip_needed = False

            #Limit to 30 frames per second but most redraws are made when needed - less often
            #30 frames per second used mainly for event handling
            self.game_board.process_ai()
            clock.tick(30)
        
        #close espeak process, collect garbage, quit pygame, and exit the game.
        self.speaker.stop_server()
        gc.collect()
        pygame.quit()
        os.sys.exit()

if __name__ == "__main__":
    #create the Thread objects and start the threads
    speaker = classes.speaker.Speaker()
    app = GamePlay(speaker)
    speaker.start()
    app.start()